<?php
include_once RP.'inc/class.BbCode.php';

class ForumPrototype extends Module {

	var $module_name = "forum";
	var $main_title;						// переопределяет title страницы, если модуль главный
	var $addition_to_path = array();		// добавляет к пути страницы строку, если модуль главный
	var $required_authorization = 1;		// добавление тем и сообщений только для авторизованных пользователей
	var $is_authorized = false;				// авторизован ли пользователь
	var $img_path = 'i/admin/forum';		// Путь к директории с изображениями пользователей

	// действия, которые пользователь имеет возможность задать для блока с данным модулем
	// если поле блока НЕ ЯВЛЯЕТСЯ полем главного модуля:
	var $block_module_actions		= array();
	// если поле блока ЯВЛЯЕТСЯ полем главного модуля:
	var $block_main_module_actions	= array();
	var $default_action				= "showforums"; // действие, выполняемое модулем по умолчанию
	var $admin_default_action		= 'showforums';
	var $tpl_path					= 'mod/forum/tpl/';
	var $forum_sates				= array();
	var $topic_sates				= array();
	var $topic_types				= array();
	var $show_for_period			= array();
	var $replies_order_by			= array();

	// Функция-конструктор
	// в зависимости от переданного $action выполняет различные действия
	function ForumPrototype($action = "", $transurl = "", $properties = array(), $prefix = NULL, $rewrite_mod_params = NULL, $adminurl = NULL, $noadmin = false) {
		global $main, $tpl, $tpl_path, $baseurl, $cache, $lc, $CONFIG, $PAGE, $server, $lang, $permissions, $request_type, $_RESULT;
		
		/* Get $action, $tpl_name, $permissions */
		extract( $this->init(array(
		        'server' => $server, 'lang' => $lang, 'prefix' => $prefix,
		        'action' => $action, 'tpl_name' => $properties['tpl']
		)), EXTR_OVERWRITE);
		
		$this->block_module_actions = array(
		        "lasttopics"	=> $this->_msg["Show_last_new_topic"],
		        "newtopics"	=> $this->_msg["Show_new_topics"],
		);
		$this->block_main_module_actions = array(
		        "showforums"	=> $this->_msg["Show_forums_list"],
		        "lasttopics"	=> $this->_msg["Show_last_new_topic"],
		        "newtopics"	=> $this->_msg["Show_new_topics"],
		);
		
		require_once('class.Smiles.php');
		$smiles = new Smiles();
		$GLOBALS['smiles'] = &$smiles;

		$this->required_authorization = $CONFIG["forum_requires_authorization"] == 1;

		// проверить авторизован ли пользователь
		$this->is_authorized = $_SESSION['siteuser']['id'] ? true : false;

		$this->forum_sates = array(
			0 => array('name' => 'open', 'title' => $this->_msg["Status_open"]),
			1 => array('name' => 'close', 'title' => $this->_msg["Status_close"]),
			2 => array('name' => 'hide', 'title' => $this->_msg["Status_hide"]),
		);
		$this->topic_sates = array(
			0 => array('name' => 'open', 'title' => $this->_msg["Status_open"]),
			1 => array('name' => 'close', 'title' => $this->_msg["Status_close"]),
		);
		$this->topic_types = array(
			0 => array('name' => 'topic', 'title' => $this->_msg["Topic"]),
			1 => array('name' => 'stick', 'title' => $this->_msg["Stick"]),
			2 => array('name' => 'notice', 'title' => $this->_msg["Notice"]),
		);
		$this->show_for_period = array(
			0 => array('title' => $this->_msg["All_time"]),
			1 => array('title' => $this->_msg["Day"]),
			7 => array('title' => $this->_msg["Week"]),
			14 => array('title' => $this->_msg["2week"]),
			30 => array('title' => $this->_msg["Month"]),
			91 => array('title' => $this->_msg["3month"]),
			183 => array('title' => $this->_msg["Half_year"]),
			365 => array('title' => $this->_msg["Year"]),
		);
		$this->replies_order_by = array(
			0 => array('title' => $this->_msg["Decrease_date"]),
			1 => array('title' => $this->_msg["Increase_date"]),
		);

		// выставим свойства, которые пользователь имеет возможность задать для блока с данным модулем
		// индекс свойства в массиве $properties соответствует номеру поля в БД
		// например, первый элемент соотв. полю property1 в БД
		if ($_SESSION['siteuser']['themes_cnt']) {
			$number_of_topics = $_SESSION['siteuser']['themes_cnt'];
		} else {
			$number_of_topics = intval($properties[1]); // показывать кол-во тем на одной странице
			if (!$number_of_topics) $number_of_topics = $CONFIG["forum_topics_max_rows"];
		}

		$number_of_replies = intval($properties[2]); // показывать кол-во ответов на тему на одной странице
		if (!$number_of_replies) $number_of_replies = $CONFIG["forum_replies_max_rows"];

		// учтем формат времени при выводе даты и времени
		if ($_SESSION['siteuser']['time_format']) {
			$CONFIG['forum_date_format'] = str_replace('%H', '%h', $CONFIG['forum_date_format']).' %p';
		}
		// учтем часовой пояс при выводе даты и времени
		if ($_SESSION['siteuser']['time_zone']) {
			$CONFIG['forum_date_format'] .= ' ('.$_SESSION['siteuser']['time_zone'].')';
		}
		
		if ('only_create_object' == $action) {
		    return;
		}
		
		if ($this->actionExists($action)) {
		    $this->doAction($action);
		    return;
		}
		
		if (! self::is_admin()){
//---------------------------------- обработка действий с сайта --------------------------------//
			$this->reg_forum_vizit((int)$GLOBALS['request_fid'], $GLOBALS['request_topic']);
			switch ($action) {
// Вывод списка форумов
			case "showforums":
				$main->include_main_blocks($this->module_name . "_list.html", "main");
				$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
				$tpl->prepare();


				if ($this->required_authorization && !$this->is_authorized)
				{
					$this->show_forum_nav((int)$fid, $topic, $start, array(5));
				}
				elseif ($this->required_authorization && $this->is_authorized)
				{
					$this->show_forum_nav((int)$fid, $topic, $start, array(6,7));
				}

				$this->show_forums();
				break;

// Вывод последних обновленных тем
			case 'lasttopics':
				$main->include_main_blocks($this->module_name.'_last_topics.html', 'main');
				$tpl->prepare();
				$this->show_last_topics($properties[1], $transurl);
				break;
				
			case 'newtopics':
				$main->include_main_blocks($this->module_name.'_new_topics.html', 'main');
				$tpl->prepare();
				$this->show_last_topics($properties[1], $transurl, 1);
				break;

// Вывод списка тем
			case "showtopics":
				global $request_fid, $request_topic, $request_parent, $request_start, $request_filter, $lang, $CONFIG;
				$fid	= (int)$request_fid;
				if (!$fid) header('Location: '.$baseurl.'&action=showforums');
				$topic	= (int)$request_topic;
				$start	= $request_start > 1 ? (int)$request_start : 1;
				$main->include_main_blocks($this->module_name.'_topics.html', 'main');
				$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
				$tpl->prepare();

				$forum_info = $this->show_forum($fid);
				if (FALSE === $forum_info) {
					header('Location: '.$baseurl.'&action=showforums');
				}

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))

					/** Форум открыт и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума**/
					|| ($this->is_authorized && $forum_info['forum_is_open'])
					|| (!$this->required_authorization && $forum_info['forum_is_open'] /*|| $forum_info["user_rights"] & 2*/))
				{
					$this->show_post_form($fid, $topic, 0, "addtopic");
				}

				//die();

				if ($this->required_authorization && !$this->is_authorized) {
					$this->show_forum_nav($fid, $topic, $start, array(0,4,5));
				} else if ($this->required_authorization && $this->is_authorized) {
					$this->show_forum_nav($fid, $topic, $start, array(0,2,4,6,7));
				} else {
					$this->show_forum_nav($fid, $topic, $start, array(0,4));
				}

				$tpl->newBlock('block_topics');

				$moderators = $forum_info['moderators'];
				unset($forum_info['moderators']);
				$tpl->assign($forum_info);
				$this->show_moderators($moderators);

				$this->showFilterForm($fid, 0, $start, $request_filter);

				$tpl->newBlock('block_themes');

				// Вывод объявлений.
				$this->show_topics($fid, $number_of_topics, 1, $forum_info["owner_id"], $request_filter, 0);

				// Вывод всего кроме объявлений.
				$pages_cnt = $this->show_topics($fid, $number_of_topics, $start, $forum_info["owner_id"], $request_filter, 1);

				$link = "action=".$action.'&fid='.$fid.$this->var_to_url_query($request_filter, "filter", "&");
				$main->_show_nav_block($pages_cnt, null, $link, $start);
				break;

			// Вывод ветки сообщений.
			case "showreplies":
				global $request_fid, $request_topic, $request_current, $request_view, $request_filter, $request_start, $lang, $request_quote, $request_edit, $server, $lang, $db;

				$main->include_main_blocks($this->module_name.'_replies.html', 'main');
				$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
				$tpl->prepare();
				$fid	= (int)$request_fid;
				$forum_info = $this->show_forum($fid);
				if (!$forum_info) header('Location: '.$baseurl.'&action=showforums');

				$topic = (int)$request_topic;
				$topic_info = $this->get_topic($topic);
				if (!$topic_info) header('Location: '.$baseurl.'&action=showtopics&fid='.$fid);

				$start	= $request_start > 1 ? (int)$request_start : 1;
				$current	= (int)$request_current ? (int)$request_current : $topic;
				$quote		= intval($request_quote);
				$edit = 0;

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))
					/** Форум и тема открыты и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума и темы**/
					|| ($topic_info['is_open'] && $forum_info['forum_is_open'] && $this->is_authorized)
					|| (!$this->required_authorization && $topic_info['is_open'] /*|| ($forum_info["user_rights"] & 2 && $topic_info["user_rights"] & 2)*/))
				{
					$this->show_post_form($fid, $topic, 0, isset($request_edit)? "editpost" : "addreply", $request_quote, $request_edit);
				}

				if ($this->required_authorization && !$this->is_authorized) {
					$this->show_forum_nav($fid, $topic, $start, array(0,1,4,5), $forum_info);
				} elseif ($this->required_authorization && $this->is_authorized) {
					$this->show_forum_nav($fid, $topic, $start, array(0,1,2,3,4,6,7), $forum_info);
				} else {
					$this->show_forum_nav($fid, $topic, $start, array(0,1,4), $forum_info);
				}

				$this->showFilterForm($fid, $topic, $start, $request_filter);

				$this->show_topic($topic, $start);
				$pages_cnt = $this->show_replies($fid, $topic, $number_of_replies, $start, $request_view, 0, $request_filter);
				$link = "action=".$action.'&fid='.$fid.'&topic='.$topic.$view_str.$this->var_to_url_query($request_filter, "filter", "&");
				$main->_show_nav_block($pages_cnt, null, $link, $start);

				break;

			case "newtopic":
				global $request_fid;
				$fid = (int)$request_fid;

				$forum_info = $this->show_forum_adm($fid);
				if (! $forum_info) {
					$main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);
					break;
				}

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))

					/** Форум открыт и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума**/
					|| ($this->is_authorized && $forum_info['forum_is_open'])
					|| (!$this->required_authorization && $forum_info['forum_is_open'] /*|| $forum_info["user_rights"] & 2*/))
				{
					$main->include_main_blocks($this->module_name.'_new.html', 'main');
					$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
					$tpl->prepare();
					
					$this->show_post_form($fid, 0, 0, 'addtopic');
					
					if ($this->required_authorization && !$this->is_authorized) {
    					$this->show_forum_nav($fid, 0, 0, array(0,1,4,5));
    				} else if ($this->required_authorization && $this->is_authorized) {
    					$this->show_forum_nav($fid, 0, 0, array(0,1,4,6,7));
    				} else {
    					$this->show_forum_nav($fid, 0, 0, array(0,1,4));
    				}
				} else {
				    $main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);
				}

				break;

			case "newreply":
				global $request_fid, $request_topic;
				$fid = (int)$request_fid;
				$topic = (int)$request_topic;

				$forum_info = $this->show_forum_adm($fid);
				if (!$forum_info) {
					$main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);
					break;
				}

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))

					/** Форум открыт и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума**/
					|| ($this->is_authorized && $forum_info['forum_is_open'])
					|| (!$this->required_authorization && $forum_info['forum_is_open'] /*|| $forum_info["user_rights"] & 2*/))
				{
					$main->include_main_blocks($this->module_name.'_new.html', 'main');
					$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
					$tpl->prepare();
					
					$this->show_post_form($fid, $topic, 0, 'addtopic');
					
					if ($this->required_authorization && !$this->is_authorized) {
    					$this->show_forum_nav($fid, $topic, 0, array(0,1,4,5));
    				} else if ($this->required_authorization && $this->is_authorized) {
    					$this->show_forum_nav($fid, $topic, 0, array(0,1,2,4,6,7));
    				} else {
    					$this->show_forum_nav($fid, $topic, 0, array(0,1,4));
    				}
				} else {
				    $main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);
				}

				break;

// Занесение в базу данных новой темы или сообщения (TOPIC или REPLY)
			case "addtopic":
				global $request_username, $request_email, $request_subject, $request_text, $request_fid, $request_topic, $request_rtm, $request_start, $request_block_id;
				$fid	= (int)$request_fid;
				$topic	= (int)$request_topic;
				$start	= $request_start >= 1 ? (int) $request_start : 1;
				$start	= $start ? $start : 1;
				$parent = 0;

				$forum_info = $this->show_forum_adm($fid);
				if (! $forum_info) {
					$main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);
					break;
				}

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))

					/** Форум открыт и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума**/
					|| ($this->is_authorized && $forum_info['forum_is_open'])
					|| (!$this->required_authorization && $forum_info['forum_is_open'] /*|| $forum_info["user_rights"] & 2*/))
				{
					$block_id = intval(substr($request_block_id, 10));
					$topic_id = $this->add_post('topic', $request_username, $request_email, $request_subject, $request_text, $fid, $topic, $parent, "", "", 0, $block_id);
					if ($topic_id) {
						header('Location: '.$baseurl.'&action=showreplies&fid='.$fid.'&topic='.$topic);
						exit;
					}
					$main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);
				}
				$main->show_result_message($this->_msg["Error_Cannot_add_new_topic"]);

				break;

			case "addreply":
				global $request_username, $request_email, $request_subject, $request_text, $request_fid, $request_topic, $request_parent, $request_rtm, $request_start;
				$fid	= (int)$request_fid;
				$topic	= (int)$request_topic;
				$start	= (int)$request_start;
				$parent	= (int)$request_parent;
				if (!$parent) $parent = $topic;

				$forum_info = $this->show_forum($fid);
				if (FALSE === $forum_info) header('Location: '.$baseurl.'&action=showforums');

				$topic_info = $this->get_topic($topic);
				if (FALSE === $topic_info) header('Location: '.$baseurl.'&action=showtopics&fid='.$fid);

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))

					/** Форум открыт и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума**/
					|| ($this->is_authorized && $forum_info['forum_is_open'] && $topic_info['is_open'])
					|| (!$this->required_authorization && $forum_info['forum_is_open'] /*|| $forum_info["user_rights"] & 2*/))
				{
					$lid = $this->add_post('reply', $request_username, $request_email, $request_subject, $request_text, $fid, $topic, $parent);
					if ($lid)
					{
						header('Location: '.$baseurl.'&action=showreplies&fid='.$fid.'&topic='.$topic);
						exit;
					}
					else
					{
						$result_message	= $this->_msg["Error_Cannot_add_new_message"];
						$forum_link		= 'javascript:history.go(-1);';
						$main->show_result_message($result_message);
						break;
					}
				}
				else
				{
					$result_message	= $this->_msg["Error_Authorization_required"];
					$forum_link		= 'javascript:history.go(-1);';
					$main->show_result_message($result_message);
					break;
				}
				break;

			case "editpost":
				global $request_subject, $request_text, $request_start, $request_id, $request_fid, $request_topic;
				$request_id	= (int)$request_id;
				$request_fid = (int)$request_fid;
				$request_topic	= (int)$request_topic;
				$request_start = $request_start >= 1 ? (int) $request_start : 1;

				$forum_info = $this->show_forum_adm($request_fid);
				if (FALSE === $forum_info) header('Location: '.$baseurl.'&action=showforums');

				$post_info = $this->get_topic($request_id);
				$topic_info = $this->get_topic($request_topic);
				if (FALSE === $post_info || FALSE === $topic_info) header('Location: '.$baseurl.'&action=showtopics&fid='.$request_fid);

				if (
					/** Пльзователь авторизован и модератор **/
					($this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']))

					/** Форум открыт и не требуется авторизация для добавления тем/сообщений или есть права на редактирование форума**/
					|| ($this->is_authorized && $forum_info['forum_is_open'] && $topic_info['is_open'] && $forum_info['edit_personal_msg']))
				{
					$this->update_reply($request_id, $request_subject, $request_text);
				}
				header('Location: '.$_SERVER['HTTP_REFERER']);
				break;

			case "delpost":
				global $request_id;
				$request_id	= (int)$request_id;

				$post_info = $this->get_topic($request_id);
				if (FALSE === $post_info) header('Location: '.$_SERVER['HTTP_REFERER']);
				$forum_info = $this->show_forum_adm($post_info['forum_id']);
				if (
					/** Пльзователь авторизован и модератор **/
					$this->is_authorized && array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					if (0 == $post_info['topic_id']) {
						$this->delete_topic($request_id);
						header('Location: '.$baseurl.'&action=showtopics&fid='.$post_info['forum_id']);
						exit;
					} else {
						$this->delete_reply($request_id);
						header('Location: '.$baseurl.'&action=showreplies&fid='.$post_info['forum_id'].'&topic='.$post_info['topic_id']);
						exit;
					}
				}

				if ($post_info['topic_id']) {
				}
				header('Location: '.$_SERVER['HTTP_REFERER']);
				break;



			case "rules":
				global $request_fid;
				$fid = ($request_fid) ? (int)$request_fid : (int)$fid;
				$start = intval($request_start);
				$main->include_main_blocks($this->module_name.'_rules.html', 'main');
				$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
				$tpl->prepare();
				/*
				if (!$this->is_authorized) {
					$tpl->assignInclude( "login_form", $tpl_path . $this->module_name . "_login_form.html" );
					$tpl->prepare();
					$tpl->assign(array(
						'form_login_action'	=>	$baseurl.'&action=login&nocash'
					));
				}
				*/
				// если форум выключен, то редирект
				if (!$this->show_forum($fid)) header('Location: '.$baseurl.'&action=showforums');

				// если на форуме включена авторизация и пользователь не авторизован
				if ($this->required_authorization && !$this->is_authorized) {
					$this->show_forum_nav($fid, $topic, $start, array(0,1,5));

				// если на форуме включена авторизация и пользователь авторизован
				} else if ($this->required_authorization && $this->is_authorized) {
					$this->show_forum_nav($fid, $topic, $start, array(0,1,6));

				// если на форуме выключена авторизация
				} else {
					$this->show_forum_nav($fid, $topic, $start, array(0));
				}

				$tpl->assign('_ROOT.forum_rules', $this->get_rules($fid));

				$this->main_title = $this->_msg["Forum_rules"];
				break;
/* Жалоба на пост */
			case 'complaint':
				global $request_id;

				$res = $this->registr_complaint($request_id);
				header("Location: ".$_SERVER['HTTP_REFERER']);
				break;

			case 'readall':
				global $request_fid, $db;

				$request_fid = (int) $request_fid;
				$topic_list = $this->get_forum_topics($request_fid);

				if ($this->is_authorized && !empty($topic_list)) {
					$query = "REPLACE INTO ".$this->table_prefix."_forum_topic_visit ";
					$query .= "(`forum_id`, `topic_id`, `user_id`, `username`, `last_visit`) VALUES";
					foreach ($topic_list as $key => $val) {
						$query .= "(".$request_fid.", ".$key.", ".$_SESSION['siteuser']['id'].", ";
						$query .= " '".$_SESSION['siteuser']['siteusername']."', DATE_ADD(NOW(), ";
						$query .= "INTERVAL ".$CONFIG['forum_is_new_time']." MINUTE)), ";
					}
					$query = substr($query, 0, -2);
					$db->query($query);
				}
				header("Location: ".$baseurl."&action=showtopics&fid=".$request_fid."&nocash");
				break;

// Default...
			default:
				if (is_object($tpl)) $tpl->prepare();
				return;
			}
//------------------------------- конец обработки действий с сайта -----------------------------//
		} else {
//------------------------------ обработка действий из админ части -----------------------------//            
			if (!isset($tpl)) $main->message_die(iconv("WINDOWS-1251", "UTF-8", 'Не подключен класс для работы с шаблонами'));

			switch ($action) {
		    case 'prp':
	    	    if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_step;
				$main->include_main_blocks('_properties.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Information"	=> $main->_msg["Information"],
					"MSG_Value"			=> $main->_msg["Value"],
					"MSG_Save"			=> $main->_msg["Save"],
					"MSG_Cancel"		=> $main->_msg["Cancel"],
				));
		        if (!$request_step || $request_step == 1) {
					$tpl->assign(array('form_action'	=> $baseurl,
									   'step'			=> 2,
									   'lang'			=> $lang,
									   'name'			=> $this->module_name,
									   'action'			=> 'prp'));
					$prp_html = $main->getModuleProperties($this->module_name);
		            $this->main_title = $this->_msg["Controls"];
				} elseif (2 == $request_step) {
		        	$main->setModuleProperties($this->module_name, $_POST);
					header('Location: '.$baseurl.'&action=prp&step=1');
				}
		    	break;

		    case 'block_prp':
				global $request_sub_action, $request_block_id;
		    	$arr  = $this->getEditLink($request_sub_action, $request_block_id);
		    	$cont = 'var container = document.getElementById(\'target_span\');
		    		  	 container.innerHTML = "'.$this->getPropertyFields().'";
		    		  	 material_id		  = "'.$arr['material_id'].'";
		    		  	 material_url		  = "'.$arr['material_url'].'";
		    		  	 changeAction();
		    	      	 setURL("property1");';
				header('Content-Length: '.strlen($cont));
		    	echo $cont;
		    	exit;
		    	break;

			case "showforums":
				global $request_err;
				if (!$permissions["r"]) $main->message_access_denied($this->module_name, $action);
				$main->include_main_blocks_2($this->module_name."_list.html", $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Activ" => $this->_msg["Activ"],
					"MSG_Ed" => $this->_msg["Ed"],
					"MSG_Rights" => $main->_msg["Rights"],
					"MSG_Forum" => $this->_msg["forum"],
					"MSG_Topics_num" => $this->_msg["Topics_num"],
					"MSG_Messages_num" => $this->_msg["Messages_num"],
					"MSG_Last" => $this->_msg["Last"],
					"MSG_Del" => $this->_msg["Del"],
					"MSG_Select_action" => $this->_msg["Select_action"],
					"MSG_Activate" => $this->_msg["Activate"],
					"MSG_Suspend" => $this->_msg["Suspend"],
					"MSG_Delete" => $this->_msg["Delete"],
					"MSG_Execute" => $this->_msg["Execute"],
					"MSG_Status" => $this->_msg["Status"],
					"MSG_Weight" => $this->_msg["Weight"],
				));

				if (isset($request_err) && 'delctg' == $request_err) {
					$tpl->newBlock('block_err_delctg');
					$tpl->assign("MSG_Cant_delete_category", $this->_msg["Cant_delete_category"]);
				}

				$this->show_forums();

				$this->main_title = $this->_msg["Forums_list"];
				break;

// Пересчет значений форума
			case "recalcforum":
				$this->update_forum_post_cnt();
				header('Location: '.$baseurl.'&action=showforums');
				exit;
				break;

// Правила форума
			case "rules":
				$main->include_main_blocks_2($this->module_name."_rules.html", $this->tpl_path);
				$tpl->assignInclude( "nav_menu", $tpl_path . $this->module_name . "_nav_menu.html" );
				global $request_fid;
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Forum" => $this->_msg["forum"],
				));
				$this->show_forum((int)$request_fid);
				$this->show_forum_nav(
					array(
						"Вернуться назад"	=>	"javascript:history.go(-1);",
						"Список форумов"	=>	"$baseurl&action=showforums"
					)
				);
				$this->main_title = $this->_msg["Forum_rules"];
				break;

// Добавление категории форумов
			case "addctg":
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_step, $request_data, $request_block_id;
				// Добавление новой записи, первый шаг
				if (!$request_step || $request_step == 1) {
					$main->include_main_blocks_2($this->module_name.'_edit_ctg.html', $this->tpl_path);
					$tpl->prepare();
					$tpl->assign(Array(
						"MSG_Enter_name" => $this->_msg["Enter_name"],
						"MSG_Name" => $this->_msg["Name"],
						"MSG_Weight" => $this->_msg["Weight"],
						"MSG_Description" => $this->_msg["Description"],
						"MSG_required_fields" => $this->_msg["required_fields"],
						"MSG_Save" => $this->_msg["Save"],
						"MSG_Cancel" => $this->_msg["Cancel"],
						'_ROOT.form_action'	=>	$baseurl.'&action=addctg&step=2',
						'_ROOT.cancel_link'	=>	$baseurl.'&action=showforums',
					));

					$this->main_title = $this->_msg["Categories"];
				} else if ($request_step == 2) {
					if ($this->set_category($request_data, intval(substr($request_block_id, 10)))) {
						@$cache->delete_cache_files();
					}
					header('Location: '.$baseurl.'&action=showforums');
					exit;
				}
				break;

// Редактирование категории форумов
               case "editctg":
                    if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
                    global $request_id;
                    if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_category", $request_id))
                                         $main->message_access_denied($this->module_name, $action);

                    $main->include_main_blocks_2($this->module_name.'_edit_ctg.html', $this->tpl_path);
                    $tpl->prepare();
                    $tpl->assign(Array(
                         "MSG_Enter_name" => $this->_msg["Enter_name"],
                         "MSG_Name" => $this->_msg["Name"],
                         "MSG_Weight" => $this->_msg["Weight"],
                         "MSG_Description" => $this->_msg["Description"],
                         "MSG_required_fields" => $this->_msg["required_fields"],
                         "MSG_Save" => $this->_msg["Save"],
                         "MSG_Cancel" => $this->_msg["Cancel"],
                    ));

                    $arr = $this->get_ctg_adm($request_id);

                    if (is_array($arr) && sizeof($arr) > 0) {                              // если нет такого форума то редирект

                         $tpl->assign(
                              array(
                                   'ctg_id'               => $arr['ctg_id'],
                                   'ctg_name'               => $arr['ctg_name'],
                                   'ctg_name'               => $arr['ctg_name'],
                                   'ctg_order'               => $arr['ctg_order'],
                                   'ctg_description'     => $arr['ctg_description'],
                                   '_ROOT.form_action'     => $baseurl.'&action=updatectg',
                                   '_ROOT.cancel_link'     => $baseurl.'&action=showforums',
                              )
                         );

                         //$this->addition_to_path[$arr['forum_name']] = $baseurl.'&action=showtopics&fid='.$arr['forum_id'];
                         $this->main_title = $this->_msg["Category"] .": ".$arr['ctg_name'];
                    } else {
                         header('Location: '.$baseurl.'&action=showforums');
                         exit;
                    }
                    break;

// Удалить категорию
			case "delctg":
				if (!$permissions["d"]) $main->message_access_denied($this->module_name, $action);
				global $request_id;
				if (!$this->test_item_rights("d", "id", $this->table_prefix."_forum_category", $request_id))
					$main->message_access_denied($this->module_name, $action);

				if ($this->del_ctg($request_id)) {
					header('Location: '.$baseurl.'&action=showforums');
					exit;
				} else {
					header('Location: '.$baseurl.'&action=showforums&err=delctg');
					exit;
				}
				break;

// Обновление категории форумов
			case "updatectg":
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_data;
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_category", $request_data['id']))
					$main->message_access_denied($this->module_name, $action);

				if ($this->set_category($request_data)) {
					@$cache->delete_cache_files();
				}

				header('Location: '.$baseurl.'&action=showforums');
				exit;
				break;

// Редактирование прав категории форумов
			case 'editctgrights':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_start;
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_category", $request_id, true))
					$main->message_access_denied($this->module_name, $action);

				if (!$this->show_rights_form($this->table_prefix."_forum_category", "name", $request_id,
											"&id=".$request_id, $this->_msg["Category"], "savectgrights")) {
					header('Location: '.$baseurl);
				}
				break;

// Сохранение прав категории форумов
			case 'savectgrights':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_start, $request_rights;
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forums", $request_id, true))
					$main->message_access_denied($this->module_name, $action);

				$this->save_rights_form($this->table_prefix."_forum_category", $request_id, $request_rights);
				if ($request_type == "JsHttpRequest") {
					$_RESULT = array(
						"res" => 1,
						"category" => $request_category,
					);
				} else {
					header('Location: '.$baseurl);
				}
				exit;
				break;

// Добавление форума в базу данных
			case "setforum":
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_step, $request_data, $request_id, $request_block_id;
				// Добавление новой записи, первый шаг
				if (!$request_step || $request_step == 1) {
					$main->include_main_blocks_2($this->module_name.'_edit.html', $this->tpl_path);
					$tpl->prepare();
					$ctg_list = $this->get_ctg_list(2);

					if (empty($ctg_list)) {
						$tpl->newBlock('block_access_deny');
						$tpl->assign(Array(
							"MSG_Add_forum_deny"	=>	$this->_msg["Add_forum_deny"],
						));
						break;
					}
					$tpl->newBlock('block_forum_form');

					$this->main_title = $this->_msg["Add_forum"];
					$forum_state = 0;
					$ctg_id = 0;
					$edit_pers_msg_chd = "checked";
					if (isset($request_id)) {
						$arr = $this->show_forum_adm($request_id);

						if (is_array($arr) && sizeof($arr) > 0) {						// если нет такого форума то редирект

							$tpl->assign(
								array(
									'text'				=> $arr['forum_rules'],
									'forum_id'			=> $arr['forum_id'],
									'forum_name'		=> $arr['forum_name'],
									'forum_description'	=> $arr['forum_description'],
									'forum_rules'		=> $arr['forum_rules'],
									'forum_state'		=> $arr['forum_state'],
									'forum_weight'		=> $arr['forum_weight'],
									'forum_link'		=> $arr['forum_link'],
								)
							);
							$forum_state = $arr['forum_state'];
							$ctg_id = $arr['category_id'];
							$this->show_select('block_moderators', 0, 'username', $arr['moderators']);
							$edit_pers_msg_chd = $arr['edit_personal_msg'] ? 'checked' : '';

							$this->main_title = $arr['forum_name'];
						}
					}
					$tpl->gotoBlock('block_forum_form');
					$tpl->assign(Array(
						"MSG_Enter_name" => $this->_msg["Enter_name"],
						"MSG_Name" => $this->_msg["Name"],
						"MSG_Description" => $this->_msg["Description"],
						"MSG_Forum_rules" => $this->_msg["Forum_rules"],
						"MSG_required_fields" => $this->_msg["required_fields"],
						"MSG_Save" => $this->_msg["Save"],
						"MSG_Cancel" => $this->_msg["Cancel"],
						"MSG_Category"	=> $this->_msg["Category"],
						"MSG_Site_users"	=> $this->_msg["Site_users"],
						"MSG_Moderators"	=> $this->_msg["Moderators"],
						"MSG_Status"	=> $this->_msg["Status"],
						"MSG_Weight"	=> $this->_msg["Weight"],
						"MSG_Edit_personal"	=> $this->_msg["Edit_personal"],
						"MSG_Fill_obligatory_fields"	=> $this->_msg["Fill_obligatory_fields"],
						"form_action"	=>	$baseurl.'&action=setforum&step=2',
						"cancel_link"	=>	$baseurl.'&action=showforums',
						"edit_personal_msg_checked"	=> $edit_pers_msg_chd,
					));

					$this->show_select('block_category', $ctg_id, 'name', $ctg_list);
					$this->show_select('block_siteusers', 0, 'username', $this->get_siteusers());
					$this->show_select('block_state', $forum_state, 'title', $this->forum_sates);
				} else if ($request_step == 2) {
					if ($this->set_forum($request_data, intval(substr($request_block_id, 10)))) {
						@$cache->delete_cache_files();
					}
					header('Location: '.$baseurl.'&action=showforums');
					exit;
				}
				break;

// Форма редактирования сообщения
			case 'editreply':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_fid, $request_topic, $request_start, $server, $lang, $db, $tpl;
				$forum_info = $this->show_forum_adm((int)$request_fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"]))
					$main->message_access_denied($this->module_name, $action);

				$id    = intval($request_id);
				$start = $request_start;

				$main->include_main_blocks_2($this->module_name.'_edit_reply.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Enter_topic_title" => $this->_msg["Enter_topic_title"],
					"MSG_Enter_topic_message" => $this->_msg["Enter_topic_message"],
					"MSG_Title" => $this->_msg["Title"],
					"MSG_Message" => $this->_msg["Message"],
					"MSG_required_fields" => $this->_msg["required_fields"],
					"MSG_Save" => $this->_msg["Save"],
					"MSG_Cancel" => $this->_msg["Cancel"],
				));
				$query = 'SELECT subject, text, forum_id, topic_id FROM '.$this->table_prefix.'_forum_posts WHERE id = '.$id;
				$db->query($query);

				if ($db->nf()){
					$db->next_record();
					$tpl->assign(array(
						'form_action' => $baseurl.'&action=updatereply',
						'id' 	      => $id,
						'start'	      => $start,
						'subject'     => $db->f('subject'),
						'text'        => $db->f('text'),
						'fid'		  => $db->f('forum_id'),
						'topic'		  => $db->f('topic_id'),
						'referer'	  => addslashes($_SERVER["HTTP_REFERER"]),
					));
				}
				$this->main_title = $this->_msg["Edit_message"];

				break;

// Редактировать сообщение
			case 'updatereply':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $db, $request_id, $request_start, $request_fid, $request_topic, $request_subject,
						$request_text, $request_referer;
				$forum_info = $this->show_forum_adm((int)$request_fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"]))
					$main->message_access_denied($this->module_name, $action);

				$id 	  = intval($request_id);
				$start	  = intval($request_start);
				$fid 	  = intval((int)$request_fid);
				$topic 	  = intval($request_topic);

				$this->update_reply($id, $request_subject, $request_text);
				header('Location: '.(empty($request_referer)
									? $baseurl.'&action=showreplies&fid='.$fid.'&topic='.$topic.'&start='.$start
									: $request_referer));
				break;

// Отключение форума
			case "suspendforum":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_id;
				if (!$this->test_item_rights("p", "id", $this->table_prefix."_forums", $request_id))
					$main->message_access_denied($this->module_name, $action);

				if ($main->suspend($this->table_prefix."_forums", $request_id)) @$cache->delete_cache_files();
				header("Location: $baseurl&action=showforums");
				exit;
				break;

// Включение форума
			case "activateforum":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_id;
				if (!$this->test_item_rights("p", "id", $this->table_prefix."_forums", $request_id))
					$main->message_access_denied($this->module_name, $action);

				if ($main->activate($this->table_prefix."_forums", $request_id)) @$cache->delete_cache_files();
				header("Location: $baseurl&action=showforums");
				exit;
				break;

// Удаляем форум
			case "delforum":
				if (!$permissions["d"]) $main->message_access_denied($this->module_name, $action);
				global $request_step, $request_id;
				if (!$this->test_item_rights("d", "id", $this->table_prefix."_forums", $request_id))
					$main->message_access_denied($this->module_name, $action);

				// Удаление категории, первый шаг
				if (!$request_step || $request_step == 1) {
					$main->include_main_blocks_2($this->module_name."_delete.html", $this->tpl_path);
					$tpl->prepare();
					$tpl->assign(Array(
						"MSG_Deleting_forum" => $this->_msg["Deleting_forum"],
						"MSG_deletes_all_its_messages" => $this->_msg["deletes_all_its_messages"],
						"MSG_Confirm_delete_this_forum" => $this->_msg["Confirm_delete_this_forum"],
						"MSG_Delete" => $this->_msg["Delete"],
						"MSG_Cancel" => $this->_msg["Cancel"],
					));
					$tpl->assign(
						array(
							"_ROOT.form_action"	=>	"$baseurl&action=delforum&step=2&id=$request_id",
							"_ROOT.cancel_link"	=>	"$baseurl&action=showforums",
						)
					);
					$this->show_forum($request_id);
					$this->main_title = $this->_msg["Delete_forum"];
				// Удаление категории, шаг второй
				} else if ($request_step == 2) {
					if ($this->delete_forum($request_id)) @$cache->delete_cache_files();
					header("Location: $baseurl&action=showforums");
					exit;
				}
				break;


// Отключить публикацию выбранных форумов
			case "suspendforum_checked":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_check;
				if(is_array($request_check)) {
					foreach($request_check as $k => $v) {
						if ($this->test_item_rights("p", "id", $this->table_prefix."_forums", $v)) {
							if ($main->suspend($this->table_prefix."_forums", $v)) @$cache->delete_cache_files();
						}
					}
				}
				header("Location: $baseurl&action=showforums");
				exit;
				break;

// Включить публикацию выбранных форумов
			case "activateforum_checked":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_check;
				if(is_array($request_check)) {
					foreach($request_check as $k => $v) {
						if ($this->test_item_rights("p", "id", $this->table_prefix."_forums", $v)) {
							if ($main->activate($this->table_prefix."_forums", $v)) @$cache->delete_cache_files();
						}
					}
				}
				header("Location: $baseurl&action=showforums");
				exit;
				break;

// Удалить выбранные форумы
			case "deleteforum_checked":
				if (!$permissions["d"]) $main->message_access_denied($this->module_name, $action);
				global $request_check;
				if(is_array($request_check)) {
					foreach($request_check as $k => $v) {
						if ($this->test_item_rights("d", "id", $this->table_prefix."_forums", $v)) {
							if ($this->delete_forum($v)) @$cache->delete_cache_files();
						}
					}
				}
				header("Location: $baseurl&action=showforums");
				exit;
				break;


// Редактирование прав категории
			case 'editforumrights':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_start;
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forums", $request_id, true))
					$main->message_access_denied($this->module_name, $action);

				if (!$this->show_rights_form($this->table_prefix."_forums", "name", $request_id,
											"&id=".$request_id, $this->_msg["forum"], "saveforumrights")) {
					header('Location: '.$baseurl);
				}
				break;

// Сохранение прав категории
			case 'saveforumrights':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_start, $request_rights;
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forums", $request_id, true))
					$main->message_access_denied($this->module_name, $action);

				$this->save_rights_form($this->table_prefix."_forums", $request_id, $request_rights);
				if ($request_type == "JsHttpRequest") {
					$_RESULT = array(
						"res" => 1,
						"category" => $request_category,
					);
				} else {
					header('Location: '.$baseurl);
				}
				exit;
				break;


// Вывод списка тем
			case "showtopics":
				if (!$permissions["r"]) $main->message_access_denied($this->module_name, $action);
				global $request_fid, $request_topic, $request_start;
				$fid = (int)$request_fid;
				if (!$fid) header('Location: '.$baseurl.'&action=showforums');
				$topic = (int)$request_topic;
				$start = (int)$request_start;
				$main->include_main_blocks_2($this->module_name.'_topics.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Choose_forum" => $this->_msg["Choose_forum"],
					"MSG_Confirm_move_topics_to" => $this->_msg["Confirm_move_topics_to"],
					"MSG_Alert_no_topics" => $this->_msg["Alert_no_topics"],
					"MSG_Forum" => $this->_msg["forum"],
					"MSG_Activ" => $this->_msg["Activ"],
					"MSG_Ed" => $this->_msg["Ed"],
					"MSG_Rights" => $main->_msg["Rights"],
					"MSG_Message_topic" => $this->_msg["Message_topic"],
					"MSG_Author" => $this->_msg["Author"],
					"MSG_Answers_num" => $this->_msg["Answers_num"],
					"MSG_Refreshed" => $this->_msg["Refreshed"],
					"MSG_Del" => $this->_msg["Del"],
					"MSG_Select_action" => $this->_msg["Select_action"],
					"MSG_Activate" => $this->_msg["Activate"],
					"MSG_Suspend" => $this->_msg["Suspend"],
					"MSG_Delete" => $this->_msg["Delete"],
					"MSG_Execute" => $this->_msg["Execute"],
					"MSG_Status" => $this->_msg["Status"],
					"MSG_Type" => $this->_msg["Type"],
					"MSG_Choose_forum_for_moving" => $this->_msg["Choose_forum_for_moving"],
					"MSG_Move" => $this->_msg["Move"],

				));
				$forum_prp = $this->show_forum_adm($fid);
				if (is_array($forum_prp)) {
					$tpl->assign(
						array(
							'forum_id' => $forum_prp['forum_id'],
							'forum_name' => $forum_prp['forum_name'],
							'forum_description' => $forum_prp['forum_description'],
							'forum_rules' => $forum_prp['forum_rules'],
							'forum_status' => $forum_prp['forum_status'],
							'forum_link' => $forum_prp['forum_link'],
							'change_forum_action' => $baseurl."&action=changeforum",
						)
					);
				}
				$pages_cnt = $this->show_topics($fid, $CONFIG['forum_topics_max_rows'], $start, $forum_prp["owner_id"]);
				$main->_show_nav_block($pages_cnt, null, "action=".$action."&fid=".$fid, $start);
                $tpl->assign(Array(
					"MSG_Pages" => $this->_msg["Pages"],
				));
				$this->show_forums();
				$this->fill_form_fields($fid, $topic, 0, $start, 'addtopic');
				$this->main_title = $this->_msg["Topics_list"];
				break;

// Вывод ветки сообщений
			case 'showreplies':
				if (!$permissions['r']) $main->message_access_denied($this->module_name, $action);
				global $request_fid, $request_topic, $request_start, $request_current;
				$fid		= (int)$request_fid;
				$topic		= (int)$request_topic;
				$start		= $request_start >= 1 ? (int)$request_start : 1;
				$current	= (int)$request_current;
				if (!$fid)		header('Location: '.$baseurl.'&action=showforums');
				if (!$topic)	header('Location: '.$baseurl.'&action=showtopics&fid='.$fid);
				if (!$current)	$current = $topic;
				$main->include_main_blocks_2($this->module_name.'_replies.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Forum" => $this->_msg["forum"],
					"MSG_Topic" => $this->_msg["Topic"],
					"MSG_Choose_topic_for_moving" => $this->_msg["Choose_topic_for_moving"],
					"MSG_Move" => $this->_msg["Move"],
					"MSG_Pages" => $this->_msg["Pages"],
					"MSG_Confirm_move_message_to" => $this->_msg["Confirm_move_message_to"],
					"MSG_Alert_no_messages" => $this->_msg["Alert_no_messages"],
					"MSG_Choose_topic" => $this->_msg["Choose_topic"],
					"change_topic_action" => $baseurl."&action=changetopic&fid=".$fid."&topic=".$topic,
				));
				$this->show_forum_nav(
					array(
						$this->_msg["Forums_list"]	=> $baseurl.'&action=showforums',
						$this->_msg["Topics_list"]		=> $baseurl.'&action=showtopics&fid='.$fid,
					)
				);

				$forum_prp = $this->show_forum_adm($fid);
				if (is_array($forum_prp)) {
					$tpl->assign(
						array(
							"forum_id" => $forum_prp['forum_id'],
							"forum_name" => $forum_prp['forum_name'],
							"forum_description" => $forum_prp['forum_description'],
							"forum_rules" => $forum_prp['forum_rules'],
							"forum_status" => $forum_prp['forum_status'],
							"forum_link" => $forum_prp['forum_link']
						)
					);
				}

				$this->show_topic($topic, $start, $forum_prp['owner_id']);
				$this->show_select("block_topics", 0, "subject", $this->get_forum_topics($fid, $topic));

				$pages_cnt = $this->show_replies($fid, $topic, $CONFIG['forum_replies_max_rows'], $start, "", $forum_prp['owner_id']);
				$main->_show_nav_block($pages_cnt, null, "action=".$action."&fid=".$fid.'&topic='.$topic, $start);
				$this->fill_form_fields($fid, $topic, $current, $start, 'addreply');

				$this->main_title = $this->_msg["Messages_list"];

				break;

// Занесение в базу данных новой темы или сообщения (TOPIC или REPLY)
			case "addtopic":
				global $request_fid, $request_topic, $request_start, $request_rtm,
						$request_username, $request_email, $request_subject, $request_text;
				$fid	= (int)$request_fid;
				$topic	= (int)$request_topic;
				$start	= (int)$request_start;
				$rtm	= (int)$request_rtm;
				$parent	= 0;

		   		if ($this->add_post('topic',
									 $request_username,
									 $request_email,
									 $request_subject,
									 $request_text,
									 $fid,
									 $topic,
									 $parent)) {@$cache->delete_cache_files();}
				header('Location: '.$baseurl.'&action=showtopics&fid=$fid');
				break;

			case "addreply":
				global $request_fid, $request_topic, $request_start, $request_rtm, $request_parent,
						$request_username, $request_email, $request_subject, $request_text;
				$fid = intval($request_fid);
				$topic = intval($request_topic);
				$start = intval($request_start);
				$rtm = intval($request_rtm);
				$parent = intval($request_parent);
				if (!$parent) $parent = $topic;
				$lid = $this->add_post('reply', $request_username, $request_email, $request_subject, $request_text, $rtm, $fid, $topic, $parent);
				if ($lid) @$cache->delete_cache_files();
				header("Location: $baseurl&action=showreplies&fid=$fid&topic=$topic&start=$start");
				break;

// Отключение темы
			case "suspendtopic":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("p", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"])
					&& !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}
				if ($main->suspend($this->table_prefix."_forum_posts", $request_id)) @$cache->delete_cache_files();
				header("Location: $baseurl&action=showtopics&fid=$request_fid&start=$request_start");
				exit;
				break;

// Включение темы
			case "activatetopic":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("p", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"])
					&& !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}
				if ($main->activate($this->table_prefix."_forum_posts", $request_id)) @$cache->delete_cache_files();
				header("Location: $baseurl&action=showtopics&fid=$request_fid&start=$request_start");
				exit;
				break;

// Удаление темы
			case "deltopic":
				if (!$permissions["d"]) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_fid, $request_start;
        $request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("d", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"])
					&& !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}

				if ($this->delete_topic($request_id)) @$cache->delete_cache_files();
				header("Location: ".((empty($_SERVER["HTTP_REFERER"]))
									? "$baseurl&action=showtopics&fid=$request_fid&start=$request_start"
									: $_SERVER["HTTP_REFERER"]));
				break;

// Отключение выбранных тем
			case "suspendtopic_checked":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_check, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				if(is_array($request_check)) {
					$forum_info = $this->show_forum_adm($request_fid);
					foreach($request_check as $k => $v) {
						if ($this->test_item_rights("p", "id", $this->table_prefix."_forum_posts", $v, false, $forum_info["owner_id"])
							|| array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
						) {
							if ($main->suspend($this->table_prefix."_forum_posts", $v)) @$cache->delete_cache_files();
						}
					}
				}
				header("Location: $baseurl&action=showtopics&fid=$request_fid&start=$request_start");
				exit;
				break;

// Включение выбранных тем
			case "activatetopic_checked":
				if (!$permissions["p"]) $main->message_access_denied($this->module_name, $action);
				global $request_check, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				if(is_array($request_check)) {
					$forum_info = $this->show_forum_adm($request_fid);
					foreach($request_check as $k => $v) {
						if ($this->test_item_rights("p", "id", $this->table_prefix."_forum_posts", $v, false, $forum_info["owner_id"])
							|| array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
						) {
							if ($main->activate($this->table_prefix."_forum_posts", $v)) @$cache->delete_cache_files();
						}
					}
				}
				header("Location: $baseurl&action=showtopics&fid=$request_fid&start=$request_start");
				exit;
				break;

// Удаление выбранных тем
			case "deletetopic_checked":
				if (!$permissions["d"]) $main->message_access_denied($this->module_name, $action);
				global $request_check, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				if(is_array($request_check)) {
					$forum_info = $this->show_forum_adm($request_fid);
					foreach($request_check as $k => $v) {
						if ($this->test_item_rights("d", "id", $this->table_prefix."_forum_posts", $v, false, $forum_info["owner_id"])
							|| array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
						) {
							if ($this->delete_topic($v)) @$cache->delete_cache_files();
						}
					}
				}
				header("Location: ".((empty($_SERVER["HTTP_REFERER"]))
									? "$baseurl&action=showtopics&fid=$request_fid&start=$request_start"
									: $_SERVER["HTTP_REFERER"]));
				break;


// Редактирование прав элемента галереи
			case 'edittopicrights':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $request_id, true, $forum_info["owner_id"]))
					$main->message_access_denied($this->module_name, $action);

				if (!$this->show_rights_form($this->table_prefix."_forum_posts", "subject", $request_id,
											"&id=".$request_id."&fid=".$request_fid."&start=".$request_start, $this->_msg["Topic"], "savetopicrights")) {
					header('Location: '.$baseurl.'&action=showtopics&fid='.$request_fid.'&start='.$request_start);
				}
				break;

// Сохранение прав элемента галереи
			case 'savetopicrights':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_fid, $request_category, $request_start, $request_rights;
				$request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $request_id, true, $forum_info["owner_id"]))
					$main->message_access_denied($this->module_name, $action);

				$this->save_rights_form($this->table_prefix."_forum_posts", $request_id, $request_rights, "topic_id");
				if ($request_type == "JsHttpRequest") {
					$_RESULT = array(
						"res" => 1,
						"id" => $request_id,
					);
				} else {
					header('Location: '.$baseurl.'&action=showtopics&fid='.$request_fid.'&start='.$request_start);
				}
				exit;
				break;


// Удаление сообщения
			case "delreply":
				if (!$permissions["d"]) $main->message_access_denied($this->module_name, $action);
				global $request_id, $request_topic, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("d", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"])
					&& !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}
				if ($this->delete_reply($request_id, $request_topic)) @$cache->delete_cache_files();
				header("Location: ".((empty($_SERVER["HTTP_REFERER"]))
									? "$baseurl&action=showreplies&fid=$request_fid&topic=$request_topic&start=$request_start"
									: $_SERVER["HTTP_REFERER"]));
				break;

// Редактирование свойств (параметров) блока
			case 'properties':
				if (!$permissions['e']) $main->message_access_denied($this->module_name, $action);
				global $module_id, $request_field, $request_block_id, $title;

				$main->include_main_blocks('_block_properties.html', 'main');
				$tpl->prepare();

				if ($main->show_actions($request_field, '', $this->block_main_module_actions, $this->block_module_actions)) {
					$main->show_linked_pages($module_id, $request_field);
				}

				$tpl->assign(array( '_ROOT.title'		=> $title,
									'_ROOT.username'	=> $_SESSION['session_login'],
									'_ROOT.password'	=> $_SESSION['session_password'],
									'_ROOT.name'		=> $this->module_name,
									'_ROOT.lang'		=> $lang,
									'_ROOT.block_id'	=> $request_block_id,
								));
				$this->main_title	= $this->_msg["Block_properties"];
				break;

// Форма редактирования темы
			case 'edittopic':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $server, $lang, $db, $tpl, $request_id, $request_fid, $request_start;
				$request_fid = (int)$request_fid;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $request_id, false, $forum_info["owner_id"])
					&& !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}

				$main->include_main_blocks_2($this->module_name.'_edit_topic.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Enter_topic_title" => $this->_msg["Enter_topic_title"],
					"MSG_Enter_topic_message" => $this->_msg["Enter_topic_message"],
					"MSG_Title" => $this->_msg["Title"],
					"MSG_Message" => $this->_msg["Message"],
					"MSG_required_fields" => $this->_msg["required_fields"],
					"MSG_Status" => $this->_msg["Status"],
					"MSG_Type" => $this->_msg["Type"],
					"MSG_Message" => $this->_msg["Message"],
					"MSG_Save" => $this->_msg["Save"],
					"MSG_Cancel" => $this->_msg["Cancel"],
				));
				$id    = intval($request_id);
				$start = $request_start;

				$query = 'SELECT username, text, subject, forum_id, status, type';
				$query .= ' FROM '.$this->table_prefix.'_forum_posts AS p';
				$query .= ' LEFT JOIN  '.$this->table_prefix.'_forum_post_state AS fs';
				$query .= ' ON fs.topic_id=p.id WHERE id = '.$id;
				$db->query($query);

				if ($db->nf()){
					$db->next_record();
					$tpl->assign(array(
						'form_action' => $baseurl.'&action=updatetopic',
						'id' 	      => $id,
						'start'	      => $start,
						'subject'     => $db->f('subject'),
						'text'        => $db->f('text'),
						'fid'		  => $db->f('forum_id'),
						'referer'	  => addslashes($_SERVER["HTTP_REFERER"]),
					));
				}
				$forum_state = (int) $db->f('status');
				$forum_type = (int) $db->f('type');
				$this->show_select('block_status', $forum_state, 'title', $this->topic_sates);
				$this->show_select('block_types', $forum_type, 'title', $this->topic_types);
				$this->main_title = $this->_msg["Edit_topic"];

				break;

// Редактировать тему
			case 'updatetopic':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $db, $request_id, $request_start, $request_fid, $request_subject,
					$request_text, $request_referer, $request_type, $request_status;
				$id = intval($request_id);
				$start = intval($request_start);
				$fid = intval((int)$request_fid);
				$forum_info = $this->show_forum_adm($fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $id, false, $forum_info["owner_id"])
				 && !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}

				$this->update_topic($id, $request_subject, $request_text, $request_type, $request_status);
				header('Location: '.(empty($request_referer)
									? $baseurl.'&action=showtopics&fid='.$fid
									: $request_referer));
				break;

// Переместить темы в другой форум
			case 'changeforum':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_new_forum, $request_check, $request_referer;
				$fid = intval((int)$request_new_forum);
				$this->move_topics($request_check, $fid);
				header('Location: '.$baseurl.'&action=showtopics&fid='.$fid);
				/*header('Location: '.(empty($request_referer)
									? $baseurl.'&action=showtopics&fid='.$fid
									: $request_referer));*/
				break;

// Переместить сообщения в другую темыу
			case 'changetopic':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_fid, $request_topic,  $request_new_topic, $request_check, $request_referer;
				$request_new_topic = (int) $request_new_topic;
				$request_fid = (int) $request_fid;
				$request_topic = (int) $request_topic;
				$forum_info = $this->show_forum_adm($request_fid);
				if (!$this->test_item_rights("e", "id", $this->table_prefix."_forum_posts", $request_fid, false, $forum_info["owner_id"])
				 && !array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators'])
				) {
					$main->message_access_denied($this->module_name, $action);
				}

				header('Location: '.
					($this->move_replies($request_check, $request_new_topic)
						&& !empty($request_referer)
					? $request_referer
					: $baseurl.'&action=showreplies&fid='.$request_fid.'&topic='.$request_topic)
				);

				break;

// Журнал администратора
			case 'log':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_start, $request_start_date, $request_end_date;
				$main->include_main_blocks_2($this->module_name.'_log.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Messages_from" => $this->_msg["Messages_from"],
					"MSG_to" => $this->_msg["to"],
					"MSG_Select" => $this->_msg["Select"],
					"MSG_Refresh_messages" => $this->_msg["Refresh_messages"],
				));

				$start	= (int)$request_start;
				$start_date	= $this->test_date($request_start_date);
				if (empty($start_date)) $start_date = mktime(12, 0, 0, date("m"), date("d")-7, date("Y"));
				$end_date	= $this->test_date($request_end_date);
				if (empty($end_date)) $end_date = mktime(12, 0, 0, date("m"), date("d"), date("Y"));
				if ($start_date > $end_date) $start_date = $end_date;
				$tpl->assign(array(
								"lang"			=> $lang,
								"start_date"	=> date("d.m.Y", $start_date),
								"end_date"		=> date("d.m.Y", $end_date),
								"baseurl"		=> $baseurl
							));

				if ($_SESSION["info_block_name"]) {
					$tpl->newBlock('block_info_box');
					$tpl->newBlock($_SESSION["info_block_name"]);
					unset($_SESSION["info_block_name"]);
				}
				$tpl->assign(Array(
					"MSG_IP_successfully_banned" => $this->_msg["IP_successfully_banned"],
				));

				$pages_cnt = $this->show_log($start_date, $end_date, $CONFIG['forum_replies_max_rows'], $start);

				$main->_show_nav_block($pages_cnt, null, "action=".$action.'&start_date='.date("d.m.Y", $start_date).'&end_date='.date("d.m.Y", $end_date), $start);
				$tpl->assign(Array(
					"MSG_Pages" => $this->_msg["Pages"],
				));

				$this->main_title = $this->_msg["Admin_logs"];
				break;

// Журнал администратора (Жалобы)
			case 'complaint':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_start, $request_start_date, $request_end_date;
				$main->include_main_blocks_2($this->module_name.'_complaint.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Messages_from" => $this->_msg["Messages_from"],
					"MSG_to" => $this->_msg["to"],
					"MSG_Select" => $this->_msg["Select"],
					"MSG_Refresh_messages" => $this->_msg["Refresh_messages"],
				));

				$start	= (int)$request_start;
				$start_date	= $this->test_date($request_start_date);
				if (empty($start_date)) $start_date = mktime(12, 0, 0, date("m"), date("d")-7, date("Y"));
				$end_date	= $this->test_date($request_end_date);
				if (empty($end_date)) $end_date = mktime(12, 0, 0, date("m"), date("d"), date("Y"));
				if ($start_date > $end_date) $start_date = $end_date;

				$tpl->assign(array(
								"lang"			=> $lang,
								"start_date"	=> date("d.m.Y", $start_date),
								"end_date"		=> date("d.m.Y", $end_date),
								"baseurl"		=> $baseurl
							));

				if ($_SESSION["info_block_name"]) {
					$tpl->newBlock('block_info_box');
					$tpl->newBlock($_SESSION["info_block_name"]);
					unset($_SESSION["info_block_name"]);
				}
				$tpl->assign(Array(
					"MSG_IP_successfully_banned" => $this->_msg["IP_successfully_banned"],
				));

				$pages_cnt = $this->show_complaints($start_date, $end_date, $CONFIG['forum_replies_max_rows'], $start);
				$main->_show_nav_block($pages_cnt, null, "action=".$action.'&start_date='.date("d.m.Y", $start_date).'&end_date='.date("d.m.Y", $end_date), $start);
				$tpl->assign(Array(
					"MSG_Pages" => $this->_msg["Pages"],
				));

				$this->main_title = $this->_msg["Admin_logs"];
				break;

// Список забаненых IP
			case 'showbanlist':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $request_start;
				$main->include_main_blocks_2($this->module_name.'_banlist.html', $this->tpl_path);
				$tpl->prepare();
				$tpl->assign(Array(
					"MSG_Del" => $this->_msg["Del"],
					"MSG_IP" => $this->_msg["IP"],
				));

				$start	= (int)$request_start ? (int)$request_start : 1;

				if ($_SESSION["info_block_name"]) {
					$tpl->newBlock('block_info_box');
					$tpl->newBlock($_SESSION["info_block_name"]);
					$tpl->assign(Array(
						"MSG_IP_moved_from_banlist" => $this->_msg["IP_moved_from_banlist"],
					));
					unset($_SESSION["info_block_name"]);
				}

				$this->show_banlist($start);
				$main->_show_nav_string($this->table_prefix.'_forum_ip_ban',
										'',
										'nav',
										'action=showbanlist',
										$start,
										$CONFIG['forum_banip_max_rows']);

				$tpl->assign(Array(
					"MSG_Pages" => $this->_msg["Pages"],
				));

				$this->main_title = $this->_msg["Baned_IP"];
				break;

// Забанить IP
			case 'banip':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $db, $request_ip;

				$this->ban_ip($request_ip);
				$_SESSION["info_block_name"] = "block_info_ban_success";
				header('Location: '.(empty($_SERVER["HTTP_REFERER"])
									? $baseurl.'&action=showbanip'
									: $_SERVER["HTTP_REFERER"]));
				break;

// Удалить забаненый IP
			case 'delbanip':
				if (!$permissions["e"]) $main->message_access_denied($this->module_name, $action);
				global $db, $request_ip, $request_start;

				$db->query("DELETE FROM ".$this->table_prefix."_forum_ip_ban WHERE ip='".$request_ip."'");
				if ($db->affected_rows()) {
					$_SESSION["info_block_name"] = "block_info_ban_delete_success";
				}
				header('Location: '.(empty($_SERVER["HTTP_REFERER"])
									? $baseurl.'&action=showbanip&start='.$request_start
									: $_SERVER["HTTP_REFERER"]));
				break;

// Создание объекта для дальнейшего использования
			case 'only_create_object':
				break;

// Default...
			default:
				$main->include_main_blocks();
				$tpl->prepare();
			}
//--------------------------- конец обработки действий из админ части --------------------------//
		}
	}


	// функция для заполнения полей ввода данными из кукизов
	// и заполнения невидимых полей данными о номере топика
	function fill_form_fields($fid, $topic, $parent, $start, $action) {
		global $CONFIG, $tpl, $baseurl;
		$sitename = $CONFIG["sitename"];
		$form_heading = ($topic) ? $this->_msg["Add_message"] : $this->_msg["New_topic"];

		if (!$this->required_authorization) {
			if ($_COOKIE["abo-forum-username"]) $username = $_COOKIE["${sitename}-forum-username"];
			if ($_COOKIE["abo-forum-email"]) $email = $_COOKIE["${sitename}-forum-email"];
		}

		$tpl->assign(
			array(
				"_ROOT.form_action"	=>	$baseurl."&action=".$action."&nocash",
				"_ROOT.fid"	=>	$fid,
				"_ROOT.topic"	=>	$topic,
//				"_ROOT.parent"	=>	$parent,
				"_ROOT.start"	=>	$start,
				"_ROOT.subject_max_length"	=>	$CONFIG["forum_subject_max_length"],
//				"_ROOT.reply"	=>	$CONFIG["forum_re"],
				"_ROOT.form_heading"	=>	$form_heading,
			)
		);

		if (!$this->required_authorization) {
			$tpl->newBlock("block_check_username");
			$tpl->newBlock("block_input_username");
			$tpl->assign(
				array(
					"username"	=>	$username,
					"email"	=>	$email,
					"username_max_length"	=>	$CONFIG["forum_username_max_length"],
					"email_max_length"	=>	$CONFIG["forum_email_max_length"],
				)
			);
		}
		if ($CONFIG["forum_send_email"]) $tpl->newBlock("block_rtm");

		return 1;
	}


	// функция для вывода форума и ответа к нему по id
	function show_forum($id = '')
	{
		global $CONFIG, $db, $db2, $tpl, $baseurl, $server, $lang;

		$id = (int)$id;
		if (!$id) {
			return FALSE;
		}
		$admin = Module::is_admin();
		$active_str = (! $admin) ? "AND active = 1" : "";

		$this->get_list_with_rights("f.*", $this->table_prefix."_forums f", "f", "f.id=".$id.' '.$active_str);

		if ($db->nf() > 0) {
			$db->next_record();
			$arr = array();
			$arr['forum_id'] = $db->f('id');
			$arr['forum_name'] = htmlspecialchars($db->f('name'));
			$arr['forum_description'] = htmlspecialchars($db->f('description'));
			$arr['category_id']			= $db->f('category_id');
			$arr['forum_rules']			= $db->f('rules');
			$arr['forum_state']			= $db->f('state');
			$arr['forum_is_open']		= (0 == $db->f('state'));
			$arr['forum_weight']		= $db->f('weight');
			$arr['forum_link']			= $baseurl.'&action=showtopics&fid='.$arr['forum_id'];

			if($tpl) $tpl->assign($arr);

			$arr['owner_id']			= $db->f('owner_id');
			$arr['user_rights']			= $db->f('user_rights');
			$arr['edit_personal_msg']	= $db->f('edit_personal_msg');
			$arr['moderators']			= $this->get_forum_moderators($id);

			$this->addition_to_path[$arr['forum_name']] = $baseurl.'&action=showtopics&fid='.$arr['forum_id'];
			$this->main_title = $arr['forum_name'];

			return $arr;
		}
		return FALSE;
	}

	function get_topic($id)
	{
		global $db;

		$id = intval($id);
		if (!$id) {
			return FALSE;
		}

		$this->get_list_with_rights(
						"p.*,
						DATE_FORMAT(post_date,'".$CONFIG["forum_date_format"]."') as postdate,
						DATE_FORMAT(updated,'".$CONFIG["forum_date_format"]."') as topic_updated,
						ib.ip as ip_ban, ps.type, ps.status, IF(0 = ps.status OR ISNULL(ps.status ), 1, 0) as is_open",
						$this->table_prefix."_forum_posts p
						LEFT JOIN ".$this->table_prefix."_forum_ip_ban ib ON p.ip = ib.ip
						LEFT JOIN ".$this->table_prefix."_forum_post_state ps ON p.ip = ps.topic_id",
						"p",
						"id = ".$id, "", "", 1, 0,
						false);
		if ($db->nf() > 0) {
			$db->next_record();
			return $db->Record;
		}
		return false;
	}


	// функция для вывода темы по id
	function show_topic($id = "", $start = "", $owner_id = 0) {
		global $CONFIG, $db, $tpl, $baseurl, $server, $lang, $smiles;

		$admin = Module::is_admin();

		$id = intval($id);
		if (!$id) {
			return FALSE;
		}
		$start = intval($start);

		$this->get_list_with_rights(
						"p.*,
						DATE_FORMAT(post_date,'".$CONFIG["forum_date_format"]."') as postdate,
						DATE_FORMAT(updated,'".$CONFIG["forum_date_format"]."') as topic_updated, ib.ip as ip_ban",
						$this->table_prefix."_forum_posts p LEFT JOIN ".$this->table_prefix."_forum_ip_ban ib ON p.ip = ib.ip", "p",
						"id = ".$id, "", "", 1, 0,
						false, $owner_id);
		if ($db->nf() > 0) {
			$db->next_record();

			$user_id = $db->f('user_id');
			$username = $db->f('username');

			// выводить или нет ссылку на профиль пользователя
			if ($this->required_authorization && $this->is_authorized && $user_id) {
				$user_info = "<a href=\"javascript:openUserWin('$baseurl&action=userinfo&id=" . $user_id . "&onlymain');\" title='" . $this->_msg["User_info"] . "'>" . $username . "</a>";
			} else if ($db->f('email')) {
				$user_info = "<a href=\"mailto:" . $db->f('email') . "\">" . $username . "</a>";
			} else {
				$user_info = $username;
			}

			if ($admin) {
				$user_info = $user_id
					? '<a href="/admin.php?lang='.$lang.'&name=siteusers&action=editsiteuser&id='.$user_id.'" title="' . $this->_msg["User_info"] . '">'.$username.'</a>'
					: $username;
			}

			$signature = ($db->f('signature')) ? "<hr>" . nl2br($db->f('signature')) : "";
			$id = $db->f('id');
			$text = $db->f('text');
			$text = str_replace('<', '&lt;', $text);
			$text = str_replace('>', '&gt;', $text);
			//$text = $smiles->parse($text);
			$text = $this->parse_misc($text);
			$ip_ban = $db->f('ip_ban');

			if (!is_object($smiles)){
				require_once('mod/forum/lib/class.Smiles.php');
				$smiles = new Smiles();
				$GLOBALS['smiles'] = &$smiles;
			}
			$text = $smiles->parse($text);

			$actions = array(	"edit"		=> "editreply",
								"delete"	=> "delreply",
								"confirm_delete"	=> $this->_msg["Confirm_delete_record"]
							);
			//вывод на экран
			$tpl->assign("subject", $db->f('subject'));
			if ($start == 0 || $start == 1) {
				$tpl->newBlock("block_topic");
				$tpl->assign(Array(
					"MSG_Author" => $this->_msg["Author"],
					"MSG_User_info" => $this->_msg["User_info"],
					"MSG_Ban_IP" => $this->_msg["Ban_IP"],
					"MSG_Date" => $this->_msg["Date"],
				));
				$tpl->assign(
					array(
						"subject"	=>	$db->f('subject'),
						"text"		=> $text,
						"signature"	=>	$signature,
						"username"	=>	$username,
						"user_info"	=>	$user_info,
						"user_link"	=>	"$baseurl&action=userinfo&id=" . $user_id,
						"replies"		=>	$db->f('replies'),
						"post_date"	=>	$db->f('postdate'),
						"updated"		=>	$db->f('topic_updated'),
						"ip"			=>	$db->f("ip"),
						"host"		=>	$db->f("host"),
					)
				);
				if ($db->f("is_owner") && empty($ip_ban)) {
					$tpl->newBlock('block_theme_ip_ban');
					$tpl->assign('banip_link', $baseurl.'&action=banip&ip='.$db->f("ip"));
				}
			}
			$this->main_title .= " &raquo; " . $db->f('subject');
			$this->addition_to_path[$db->f('subject')] = "";

			$author_module = $db->f('author_module');
			if ($author_module) {
				$mod_link = Core::formPageLink($CONFIG[$author_module.'_page_link'], $CONFIG[$author_module.'_page_address'], $lang, 1);
				$mod_link .= $db->f('author_module');
				$tpl->newBlock('block_author_module');
				$tpl->assign(array(
					'author_module_link' => $mod_link,  //$_SERVER['HTTP_REFERER']
					'site_name'			 => $_SERVER["HTTP_HOST"],
				));
			}

			return 1;
		}
		return 0;
	}

	function show_forums()
	{
		global $CONFIG, $db, $db2, $tpl, $baseurl, $permissions;

		$admin = Module::is_admin();
		if ($permissions['e']) {
			$tpl->newBlock("block_add");
			$tpl->assign(
				array(
					'add_ctg_link' => $baseurl."&action=addctg",
					'add_forum_link' => $baseurl."&action=setforum",
					'recalc_forum_link' => $baseurl."&action=recalcforum",
					'MSG_Add_category' => $this->_msg["Add_category"],
					'MSG_Add_forum' => $this->_msg["Add_forum"],
					'MSG_Recalculate_forum_info' => $this->_msg["Recalculate_forum_info"],
				)
			);
		}
		
		Module::load_mod('siteusers', 'only_create_object');
		
		$this->get_list_with_rights("c.*", $this->table_prefix."_forum_category c", "c", "", "", "c.order");
		if (!$db->nf() > 0) {
			return false;
		}

		// Список категорий.
		$ctg_list = array();
		while($db->next_record())
		{
			$ctg_list[$db->f("id")] = array(
				'title' => $db->f("name"),
				'description' => $db->f("description"),
				'order' => $db->f("order"),
				'is_owner' => $db->f("is_owner"),
				'user_rights' => $db->f("user_rights"),
				'forum_list' => array()
			);
		}
		$where = $admin ? '' : ' f.active = 1 AND f.state < 2';


		$this->get_list_with_rights(
			"f.*, f.replies AS posts, COUNT(DISTINCT ou.sid) AS online_users,
			 p.subject AS last_post_subject,
			 DATE_FORMAT(last_post_date,'".$CONFIG['forum_date_format']."') AS last_date,
			 ".(!$this->is_authorized ? "0  AS visit_topics, 0 AS has_new" : "COUNT(tv.topic_id) AS visit_topics,
						IF (MIN(tv.last_visit) < DATE_ADD(last_post_date, INTERVAL ".$CONFIG['forum_is_new_time']." MINUTE), 1, 0) AS has_new"
				),
			$this->table_prefix."_forums AS f
			LEFT JOIN ".$this->table_prefix."_forum_online_user AS ou ON ou.forum_id=f.id"
			.($this->is_authorized ? " LEFT JOIN ".$this->table_prefix."_forum_topic_visit AS tv ON tv.forum_id=f.id AND tv.user_id=".$_SESSION['siteuser']['id'] : "")
			." LEFT JOIN ".$this->table_prefix."_forum_posts AS p ON p.id=f.last_post_id",
			"f",
			$where,
			"f.id",
			"f.weight"
		);


		// Список форумов.
		while($db->next_record())
		{
			$ctg_list[$db->f("category_id")]['forum_list'][] = $db->Record;
		}


		$actions = array(
			"edit"		=> "setforum",
			"editrights"=> "editforumrights",
			"activate"	=>	array("activateforum","suspendforum"),
			"delete"	=> "delforum",
			"confirm_delete"	=> $this->_msg["Confirm_delete_this_forum"]
		);
		$ctg_actions = array(
			"edit"		=> "editctg",
			"editrights"=> "editctgrights",
			"delete"	=> "delctg",
			"confirm_delete"	=>  $this->_msg["Confirm_delete_this_category"]
		);


		$last_ctg_id = 0;
		$tpl->newBlock("block_forum_list");
		foreach ($ctg_list as $ctg_id => $ctg) {
			if (!$admin && sizeof($ctg['forum_list']) == 0) continue;

			$adm_actions = $this->get_actions_by_rights(
				$baseurl."&id=".$ctg_id,
				1,
				$ctg['is_owner'],
				$ctg['user_rights'],
				$ctg_actions
			);
			$tpl->newBlock("block_category");
			$tpl->assign(
				array(
					'category_edit_action' => $adm_actions['edit_action'],
					'category_edit_rights_action' => $adm_actions['edit_rights_action'],
					'category_del_action' => $adm_actions['delete_action'],
					'category_title' =>	$ctg['title'],
					'category_order' =>	$ctg['order'],
					'forum_description' => $ctg['description']
				)
			);
			foreach ($ctg['forum_list'] as $key => $forum)
			{
				$adm_actions = $this->get_actions_by_rights(
					$baseurl."&id=".$forum['id'],
					$forum['active'],
					$forum['is_owner'],
					$forum['user_rights'],
					$actions
				);

				$has_new = $this->is_authorized && ( $forum['has_new'] ||  $forum['visit_topics'] < $forum['topics']) ? 1 : 0;
				$bgcolor = ($bgcolor == "#FFFFFF" || $bgcolor == "") ? "#FFFFFF" : "#FFFFFF";
				$tpl->newBlock("block_forum");
				$tpl->assign(
						array(
							'forum_id'					=>	$forum['id'],
							'forum_name'				=>	$forum['name'],
							'forum_description'			=>	$forum['description'],
							'forum_topics'				=>	(int)$forum['topics'],
							'forum_posts'				=>	(int)$forum['posts'],
							'forum_state'				=>	$this->forum_sates[$forum['state']]['name'],
							'forum_new'					=>	$has_new ? 'new' : '',
							'forum_weight'				=>	$forum['weight'],
							'forum_last_date'			=>	$forum['last_date'],
							'forum_last_user_id'		=>	$forum['last_user_id'],
							'forum_last_username'		=>	$forum['last_username'],
							'online_users'				=>	$forum['online_users'],
							'online_users_no'			=>	($forum['online_users'] > 0) ? "" : "-no",
							'forum_edit_action'			=>	$adm_actions["edit_action"],
							'forum_edit_rights_action'	=>	$adm_actions["edit_rights_action"],
							'forum_del_action'			=>	$adm_actions["delete_action"],
							'change_status_action'		=>	$adm_actions["change_status_action"],
							'forum_link'				=>	$baseurl."&action=showtopics&fid=".$forum['id'],
							'bgcolor'					=>	$bgcolor,
						)
				);


				if ($forum['last_date'])
				{
					$tpl->newBlock('block_last_post');
					$tpl->assign(
						array(
							'forum_last_date'		=> $forum['last_date'],
							'forum_last_subject'	=>	$forum['last_post_subject'],
						)
					);
					if ($forum['last_user_id']) {
						$tpl->newBlock('block_auth_user');
						$tpl->assign(
							array(
								'forum_last_username' => $forum['last_username'],
								'user_link' => SiteusersPrototype::getUserLink($forum['last_username']),
							)
						);
					} else {
						$tpl->newBlock('block_no_auth_user');
						$tpl->assign(
							array(
								'forum_last_username' => $forum['last_username'],
							)
						);
					}
				}
				if ($adm_actions["rights"]["d"] || $adm_actions["rights"]["p"]) {
					$tpl->newBlock('block_check');
					$tpl->assign("forum_id", $forum['id']);
				}
			}
		}

		return 1;
	}

	/**
	 * Enter description here...
	 *
	 * @param int $fid
	 * @param int $rows
	 * @param int $start
	 * @param int $forum_owner_id
	 * @param array $filter
	 * @param mixed $show_all - true - все темы, 0 - только объявления, 1 - кроме объявлений
	 * @return unknown
	 */
	function show_topics($fid, $rows, $start, $forum_owner_id = 0, $filter = FALSE, $show_all = true)
	{
		global $CONFIG, $main, $db, $tpl, $baseurl, $server, $lang;

		$fid = intval($fid); // id форума
		if (!$fid) {
			return 0;
		}
		$admin = Module::is_admin();
		if (!$rows) { // выводить строк
			$rows = $CONFIG["forum_topics_max_rows"];
		}
		Module::load_mod('siteusers', 'only_create_object');
		$order_by = "fs.type DESC, ".$CONFIG["forum_topics_order_by"]; // порядок вывода
		$start = (intval($start)) ? $start : 1; // выводить начиная со страницы

		// Если работаем не в backoffice, то показывать только "активные" темы.
		$where = $admin ? "" : " AND fp.active = 1";

		$preff = '';
		if (0 === $show_all) {
			// Отображать только объявления.
			$where .= " AND fs.type=2";
			$preff = '_notice';
		} else if (1 === $show_all) {
			// Отображать всё, кроме объявлений.
			$where .= " AND (fs.type IS NULL OR fs.type!=2)";
		}

		$preff = '';

		if (isset($filter['period']) && $filter['period'] > 0) {
			$where .= " AND fp.updated > DATE_SUB(NOW(), INTERVAL ".intval($filter['period'])." DAY)";
		}
		$total_cnt = $this->get_list_with_rights(
			"fp.*, fs.status, fs.type,
				DATE_FORMAT(fp.post_date,'".$CONFIG["forum_date_format"]."') as postdate,
				DATE_FORMAT(fp.updated,'" . $CONFIG["forum_date_format"]."') as topic_updated,
				lr.user_id as last_user_id, lr.username as last_username,
				DATE_FORMAT(lr.post_date,'".$CONFIG["forum_date_format"]."') as last_post_date,"
				.(!$this->is_authorized
					? "0 AS has_new"
					: "IF (
						ISNULL(tv.topic_id)
						OR tv.last_visit < DATE_ADD(lr.post_date, INTERVAL ".$CONFIG['forum_is_new_time']." MINUTE)
						OR tv.last_visit < DATE_ADD(fp.post_date, INTERVAL ".$CONFIG['forum_is_new_time']." MINUTE)
						, 1, 0) AS has_new"
				),
			$this->table_prefix."_forum_posts fp
				LEFT JOIN ".$this->table_prefix."_forum_post_state fs ON fp.id=fs.topic_id
				LEFT JOIN ".$this->table_prefix."_forum_last_reply lr ON fp.id=lr.topic_id"
				.($this->is_authorized
					? " LEFT JOIN ".$this->table_prefix."_forum_topic_visit AS tv ON tv.topic_id=fp.id AND tv.user_id=".$_SESSION['siteuser']['id']
					: ""),
			"fp",
			"fp.forum_id = ".$fid." AND fp.topic_id = 0 ".$where,
			"",
			$order_by,
			$start,
			$rows,
			true,
			$forum_owner_id
		);
		$pages_cnt = ceil($total_cnt/$rows);
		$new_cnt = 0;		
		if ($db->nf() > 0) {
			$tpl->gotoBlock('block_themes');
			$actions = array(	"edit"		=> "edittopic",
								"editrights"=> "edittopicrights",
								"activate"	=>	array("activatetopic","suspendtopic"),
								"delete"	=> "deltopic",
								"confirm_delete"	=> $this->_msg["Confirm_delete_this_topic"]
							);
			// вывод на экран списка
			while($db->next_record())
			{
				$user_id = $db->f('user_id');
				$username = $db->f('username');

				// выводить или нет ссылку на профиль пользователя
				$link = SiteusersPrototype::getUserLink($username);
				if ($this->required_authorization && $this->is_authorized && $user_id) {					
					$user_info = '<a href="'.$link.'" title="' . $this->_msg["User_info"] . '">'.$username.'</a>';
				} else if ($db->f('email')) {
					$user_info = "<a href=\"mailto:" . $db->f('email') . "\">" . $username . "</a>";
				} else {
					$user_info = $db->f('username');
				}

				if ($admin) {
					$user_info = $user_id
						? '<a href="/admin.php?lang='.$lang.'&name=siteusers&action=editsiteuser&id='.$user_id.'" title="' . $this->_msg["User_info"] . '">'.$username.'</a>'
						: $username;
				}

				$signature = ($db->f('signature')) ? "<hr>" . nl2br($db->f('signature')) : "";
				$id = $db->f('id');
				$bgcolor = ($bgcolor == "#FFFFFF" || $bgcolor == "") ? "#FFFFFF" : "#FFFFFF";
				$alt_text = $main->tokenize(
					str_replace(
						"'",
						'"',
						htmlspecialchars(preg_replace("/\[.+\]/U", "", $db->f('text')))
					),
					50
				);

				$adm_actions = $this->get_actions_by_rights($baseurl."&fid=".$fid."&id=".$id."&start=".$start, $db->f("active"), $db->f("is_owner"), $db->f("user_rights"), $actions);

				$type = (int) $db->f('type');
				$status = (int) $db->f('status');
				$replies =	$db->f('replies');
				$last_user_id = $db->f('last_user_id');
				$last_username = $db->f('last_username');
				$postdate = $db->f('postdate');
				$last_post_date = $db->f('last_post_date');
				if (! $last_post_date) {
					$last_post_date = $postdate;
					$last_username = $username;
					$last_user_id = $user_id;
				}
				$has_new = $db->f('has_new') ? 'new' : '';
				if ($has_new) {
					$new_cnt++;
				}
				$tpl->newBlock("block".$preff."_row");
				$type = $replies >= $CONFIG['forum_hot_count'] && 0 == $type? 'hot' : $this->topic_types[$type]['name'];
				$params = array(
					'topic_id'			=>	$id,
					'subject'			=>	$db->f('subject'),
					'alt_text'			=>	$alt_text,
					'text'				=>	nl2br($db->f('text')),
					'signature'			=>	$signature,
					'username'			=>	$username,
					'user_info'			=>	$user_info,
					'user_link'			=>	$link,
					'replies'			=>	$db->f('replies'),
					'post_date'			=>	$postdate,
					'updated'			=>	$db->f('topic_updated'),
					'views'				=>	$db->f('views'),
					'last_post_date'	=>	$last_post_date,
					'bgcolor'			=>	$bgcolor,
					'type'				=>	$type,
					'status'			=>	$this->topic_sates[$status]['name'],
					'new'				=>	$has_new,
					'new_message'		=>	($db->f('new_message')) ? "<span class=fnew>новое!</span>&nbsp;&nbsp;" : "",
					'only_new'			=>	($db->f('have_new')) ? "<img src=\"/i/i_red.gif\" width=\"8\" height=\"9\" align=\"absmiddle\" alt=\"\">" : "",
					'topic_link'		=>	$baseurl."&action=showreplies&fid=".$fid."&topic=".$id,
					'topic_edit_action'	=>	$adm_actions["edit_action"],
					'topic_edit_rights_action'	=>	$adm_actions["edit_rights_action"],
					'topic_del_action'	=>	$adm_actions["delete_action"],
					'change_status_action'		=>	$adm_actions["change_status_action"],
				);

				//print_r($params);
				//print "<br /><br />";
				$tpl->assign($params);
				if ($user_id) {
					$tpl->newBlock('block'.$preff.'_auth_author');
					$tpl->assign(
						array(
							'user_link' => $link,
							'username'	=> $username
						)
					);
				} else {
					$tpl->newBlock('block'.$preff.'_no_auth_author');
					$tpl->assign(
						array(
							'username'	=> $username
						)
					);
				}

				if ($last_user_id) {
					$tpl->newBlock('block'.$preff.'_auth_last_user');
					$tpl->assign(
						array(
							'user_link' => SiteusersPrototype::getUserLink($last_username),
							'username'	=> $last_username
						)
					);
				} else {
					$tpl->newBlock('block'.$preff.'_no_auth_last_user');
					$tpl->assign(
						array(
							'username'	=> $last_username
						)
					);
				}

				if ($adm_actions["rights"]["d"] || $adm_actions["rights"]["p"]) {
					$tpl->newBlock('block'.$preff.'_topic_check');
					$tpl->assign("topic_id", $id);
				}


				$p_cnt = ceil($replies/$CONFIG['forum_replies_max_rows']);
				$main->_show_nav_block($p_cnt, $preff ? "notice_nav" : "topic_nav", "action=showreplies&fid=".$fid."&topic=".$id, $p_cnt + 1);
			}

			if ($new_cnt > 0) {
				$tpl->newBlock('block'.$preff.'_has_new');
				$tpl->assign('read_all_link', $baseurl."&action=readall&fid=".$fid);
			}
		} else if (0 !== $show_all) {
			$tpl->newBlock('block_no_themes');
		}

		return $pages_cnt;
	}

	function get_forum_topics($forum_id, $ignor = 0)
	{
		global $db, $CONFIG;

		$query = "SELECT id, subject FROM ".$this->table_prefix."_forum_posts ";
		$query .= "WHERE forum_id=".(int) $forum_id." AND topic_id=0";
		$query .= ($ignor >= 1 ? " AND id!=".intval($ignor) : "")." ORDER BY ".$CONFIG["forum_topics_order_by"];
		$db->query($query);
		$res = array();
		while ($db->next_record()) {
			$res[$db->f('id')] = array('subject' => $db->f('subject'));
		}

		return $res;
	}


	// функция для вывода последних обновленных тем
	function show_last_topics($number_of_topics, $transurl, $new = false) {
		global $CONFIG, $main, $db, $tpl, $server, $lang;
		$rows = $number_of_topics;
		if (!$rows) $rows = $CONFIG["forum_last_topics_max_rows"]; // выводить строк
		$order_by = "ORDER BY ".($new ? 'post_date' : 'updated')." DESC"; // порядок вывода

		$query = "SELECT ".$this->table_prefix."_forum_posts.*,
						 ".$this->table_prefix."_forums.name as fname,
						 DATE_FORMAT(updated,'".$CONFIG["forum_date_format"]."') as topic_updated
					FROM ".$this->table_prefix."_forum_posts,
						 ".$this->table_prefix."_forums
					WHERE topic_id = 0 AND
						  ".$this->table_prefix."_forum_posts.active = 1 AND
						  ".$this->table_prefix."_forums.id = forum_id AND
						  ".$this->table_prefix."_forums.active = 1
						  $order_by
					LIMIT 0,".$rows;
		$db->query($query);
		if ($db->nf() > 0) {
			// вывод на экран списка
			while($db->next()) {
				$user_info = $db->f('username');
				$signature = ($db->f('signature')) ? "<hr>" . nl2br($db->f('signature')) : "";
				$id = $db->f('id');
				$fid = $db->f('forum_id');
				$bgcolor = ($bgcolor == "#FFFFFF" || $bgcolor == "") ? "#FFFFFF" : "#FFFFFF";
				$alt_text = $main->tokenize(str_replace("'", '"', $db->f('text')), 50);

				$tpl->newBlock("block_row");
				$tpl->assign(
					array(
						subject		=> $db->f('subject'),
						forum_name	=> $db->f('fname'),
						alt_text	=> $alt_text,
						text		=> nl2br($db->f('text')),
						signature	=> $signature,
						username	=> $db->f('username'),
						user_info	=> $user_info,
						replies		=> $db->f('replies'),
						post_date	=> $db->f('postdate'),
						updated		=> $db->f('topic_updated'),
						bgcolor		=> $bgcolor,
						topic_link	=> $transurl.'&action=showreplies&fid='.$fid.'&topic='.$id,
					)
				);
			}
			return $db->nf();
		}
		return 0;
	}


	// функция для вывода списка сообщений
	function show_replies($fid, $topic, $rows, $start, $view = "", $owner_id = 0, $filter = array())
	{
		global $CONFIG, $main, $db, $db1, $db2, $tpl, $baseurl, $server, $lang, $smiles;

		$fid = (int)$fid;												// id форума
		$topic = (int)$topic;												// id темы

		$forum_info = $this->show_forum($fid);
		$topic_info = $this->get_topic($topic);
		if (!$topic_info || !$forum_info) {
        	return FALSE;
        }
        $is_moder = array_key_exists($_SESSION['siteuser']['id'], $forum_info['moderators']);
		$admin = Module::is_admin();
    	$rows = $rows > 1 ? (int) $rows : $CONFIG["forum_replies_max_rows"];	// выводить строк
    	$order_by = isset($filter['order'])
    		? "post_date " . ($filter['order'] ? 'ASC' : 'DESC')
    		: $CONFIG["forum_replies_order_by"];						// порядок вывода
		$start = $start > 1 ? (int)$start : 1;								// выводить начиная со страницы

		// строчка с условием проверки "является ли сообщение новым" (появилось после последнего посещения)
		if ($this->is_authorized && $_SESSION['siteuser']['last_visit']) {
			$is_new_message = " ,IF(UNIX_TIMESTAMP(post_date) > UNIX_TIMESTAMP('" . $_SESSION["siteuser"]["last_visit"] . "'), 1, 0) as new_message ";
		}

		// показывать только новые сообщения:
			$where = $view == "onlynew" && $_SESSION["siteuser"]["last_visit"]
			? " AND (post_date > '" . $_SESSION["siteuser"]["last_visit"] . "')" : "";

		// если работаем не в backoffice, то показывать только "активные" сообщения
		$where .= $admin ? "" : " AND P.active = 1";
		if (isset($filter['period']) && $filter['period'] > 0) {
			$where .= " AND P.post_date > DATE_SUB(NOW(), INTERVAL ".intval($filter['period'])." DAY)";
		}
		
		Module::load_mod('siteusers', 'only_create_object');

		$total_cnt = $this->get_list_with_rights(
			'P.*, U.img as user_img, U.last_visit, DATE_FORMAT(post_date,"'.$CONFIG['forum_date_format'].'") as postdate, '
			    .'DATE_FORMAT(updated,"'.$CONFIG['forum_date_format'].'") as topic_updated, IB.ip as ip_ban '
			    .$is_new_message.' '.$topic_have_new,
			$this->table_prefix."_forum_posts AS P LEFT JOIN ".$this->table_prefix."_forum_ip_ban IB ON P.ip = IB.ip"
			    ." LEFT JOIN core_users U ON U.id = P.user_id",
			"P", "forum_id = ".$fid.' AND (topic_id = '.$topic.' OR P.id = '.$topic.') '.$where,
			"", $order_by, $start, $rows, true, $owner_id
		);
		$pages_cnt = ceil($total_cnt/$rows);
		if ($db->nf() > 0) {
			$actions = array(	"edit"		=> "editreply",
								"delete"	=> "delreply",
								"confirm_delete"	=> $this->_msg["Confirm_delete_this_message"]
							);
			// вывод на экран списка
			while($db->next_record()) {

				$user_id = $db->f('user_id');
				$username = $db->f('username');
				$topic_id = $db->f('topic_id');
				$is_owner = $db->f('is_owner');

				// выводить или нет ссылку на профиль пользователя
				$link = SiteusersPrototype::getUserLink($username);
				if ($user_id) {
					$user_info = '<a href="'.$link.'" title="' . $this->_msg["User_info"] . '">'.$username.'</a>';
				} else {
					$user_info = $username;
				}

				if ($admin) {
					$user_info = $user_id
						? '<a href="/admin.php?lang='.$lang.'&name=siteusers&action=editsiteuser&id='.$user_id.'" title="' . $this->_msg["User_info"] . '">'.$username.'</a>'
						: $username;
				}

				$signature	= ($db->f('signature')) ? nl2br($db->f('signature')) : "";
				$id			= $db->f('id');
				$bgcolor	= ($bgcolor == "#FFFFFF" || $bgcolor == "") ? "#FFFFFF" : "#FFFFFF";

				$text = $db->f('text');
				$text = str_replace('<', '&lt;', $text);
				$text = str_replace('>', '&gt;', $text);

				$subject = $db->f('subject');
				$postdate = $db->f('postdate');
				$topic_updated = $db->f('topic_updated');
				$new_message = $db->f('new_message');
				$ip = $db->f('ip');
				$ip_ban = $db->f('ip_ban');
				$edit_user_id = $db->f('edit_user_id');
				$edit_username = $db->f('edit_username');

				$text = $smiles->parse($text);
				$text = $this->parse_misc($text);
				$query = 'SELECT COUNT(id) AS user_messages FROM '.$this->table_prefix.'_forum_posts WHERE user_id = '.$user_id.' AND user_id <> 0';
				$db1->query($query);

				if ($db1->nf()){
					$db1->next_record();
					$user_messages = $db1->f('user_messages');
				} else {
					$user_messages = 0;
				}
				$img = $db->Record['user_img'] ? '<img src="/'.$CONFIG['siteusers_img_path'].$db->Record['user_img'].'" alt="" border="0" hspace="10" vspace="10" /><br clear="all" />' : '';
				$user_last_visit = $db->Record['last_visit'];
				$authority = "";
				$adm_actions = $this->get_actions_by_rights($baseurl."&fid=".$fid."&topic=".$topic."&id=".$id."&start=".$start, $db->f("active"), $db->f("is_owner"), $db->f("user_rights"), $actions);

				$tpl->newBlock('block_row');
				$tpl->assign(Array(
					"MSG_Author" 		=> $this->_msg["Author"],
					"MSG_User_info" 	=> $this->_msg["User_info"],
					"MSG_Ban_IP" 		=> $this->_msg["Ban_IP"],
					"MSG_Title" 		=> $this->_msg["Title"],

					"subject"			=> $subject,
					"text"				=> $text,
					"signature"			=> $signature,
					"username"			=> $username,
					"user_info"			=> $user_info,
					"user_link"			=> $link,
					"post_date"			=> $postdate,
					"updated"			=> $topic_updated,
					"bgcolor"			=> $bgcolor,
					"new_message"		=> ($new_message) ? '<span class=fnew>' . $this->_msg["new"] . '</span>&nbsp;&nbsp;' : '',
					"user_messages"		=> $user_messages,
					"quote_reply_link"	=> $baseurl.'&action=showreplies&fid='.$fid.'&topic='.$topic.'&id='.$id.'&start='.$start.'&quote='.$id.$this->var_to_url_query($filter, "filter", "&"),
					"user_last_visit"	=> $user_last_visit,
					"reply_del_action"	=> $adm_actions["delete_action"],
					"reply_edit_action"	=> $adm_actions["edit_action"],
					"img"				=> $img,
					"authority"         => $authority
				));
				
				if ($img) {
				    $tpl->newBlock('user_img');					
				} else {
				    $tpl->newBlock('user_no_img');
				}
				$tpl->assign(Array(
					"username"			=> $username,
					"user_info"			=> $user_info,
					"user_link"			=> $link,
					"user_last_visit"	=> $user_last_visit,
					"img"				=> $img,
					'img_path' => '/'.$CONFIG['siteusers_img_path'].$db->Record['user_img']
				));
				if ($is_moder && empty($ip_ban)) {
					$tpl->newBlock('block_ip_ban');
					$tpl->assign('banip_link', $baseurl.'&action=banip&ip='.$ip);
				}
				if ($user_id) {
					$tpl->newBlock('block_auth_user');
					$tpl->assign(
						array(
							"img" => $img,
							"user_messages" => $user_messages,
							"user_last_visit"	=> $user_last_visit,
							"authority"         => $user_data['authority'],
						)
					);
				}
				if (0 != $topic_id) {
					$tpl->newBlock('block_is_reply');
					$tpl->assign('id', $id);
				}
				if ($this->is_authorized) {
					$tpl->newBlock('block_complaint_link');
					$tpl->assign('complaint_link', $baseurl.'&action=complaint&id='.$id);
				}
				if ($edit_user_id > 0)
				{
					$tpl->newBlock('block_was_edit');
					$tpl->assign(
						array(
							"username" => $edit_username,
							"user_link" => SiteusersPrototype::getUserLink($edit_username),
							"updated" => $topic_updated
						)
					);
				}

				// Редктировать можно только в случае, если пользователь модератор или
				// автор сообщения, при условии, что форум и тема открыты и разрешено редактирование сообщений,
				// гости не могут редактировать сообщения.
				if ($is_moder || ($this->is_authorized && $forum_info['edit_personal_msg'] && $is_owner && $topic_info['is_open'] && $forum_info['forum_is_open']))
				{
					$tpl->newBlock('block_edit_link');
					$tpl->assign('edit_link', $baseurl.'&action=showreplies&fid='.$fid.'&topic='.$topic.'&id='.$id.'&start='.$start.'&edit='.$id.$this->var_to_url_query($filter, "filter", "&"));
				}

				if ($is_moder)
				{
					$tpl->newBlock('block_del_link');
					$tpl->assign('del_link', $baseurl.'&action=delpost&id='.$id);
				}

				if ($signature)
				{
					$tpl->newBlock('block_signature');
					$tpl->assign('signature', $signature);
				}
			}
			$db->query("UPDATE ".$this->table_prefix."_forum_posts SET views=views+1 WHERE id=".$topic);
			if (! $admin && $this->is_authorized) {
				$db->query("REPLACE INTO ".$this->table_prefix."_forum_topic_visit
				    (forum_id, topic_id, user_id, username, last_visit) VALUES
				    (".$fid.", ".$topic.", ".$_SESSION['siteuser']['id'].",
				     '".$_SESSION['siteuser']['siteusername']."', NOW())"
				);
			}
			return $pages_cnt;
		}
		return FALSE;
	}


	// функция для добавления сообщений в БД
	// если type = topic, то это сообщение - новая тема
	// темы, как и ответы на них, записываются в одну и ту же таблицу
	// только у тем parent_id и topic_id = 0
	function add_post($type, $username, $email, $subject, $text, $fid, $topic, $parent,
						$author_module="", $author_module_link="", $item_rights=0, $block_id=0) {
		global $PAGE, $CONFIG, $db, $baseurl, $server, $lang;
		$fid	= (int)$fid;
		$topic	= (int)$topic;
		$parent = (int)$parent;

		if (!$fid) return FALSE;
		if ($type == 'reply' && !$topic) return FALSE;

		if ($this->is_authorized) {
			$user_id	= intval($_SESSION['siteuser']["id"]);
			$signature	= $db->escape($_SESSION['siteuser']["signature"]);
			$username	= $db->escape($_SESSION['siteuser']["siteusername"]);
			$email		= $db->escape($_SESSION['siteuser']["email"]);
		} else {
			$user_id	= 0;
			$username	= $username ? $username : $CONFIG['forum_default_username'];
			//$username	= $db->escape(strip_tags(substr($username, 0, $CONFIG["forum_username_max_length"])));
			$username	= $CONFIG['forum_default_username'];
			$email		= $db->escape(strip_tags(substr($email, 0, $CONFIG["forum_email_max_length"])));
			$signature	= '';
		}

		// обработать входные данные
		$this->init_censor_array();
		$subject = $db->escape(strip_tags(substr($subject,0, $CONFIG["forum_subject_max_length"])));

		$this->censor_parse($subject);
		$text = htmlspecialchars(strip_tags($text));
		$text = $db->escape(substr($text, 0, $CONFIG["forum_text_max_length"]));
		$this->censor_parse($text);
		
		$ip		= $_SERVER['REMOTE_ADDR'];
		$host	= $_SERVER['REMOTE_HOST'];
		$db->query('SELECT count(*) as cnt FROM '.$this->table_prefix.'_forum_ip_ban WHERE ip="'.$ip.'"');
		$db->next_record();
		if ($db->f('cnt')) return FALSE;

		$sitename = $CONFIG["sitename"];
		$owner_id = intval($_SESSION["siteuser"]["id"]);
		$id = "NULL";
		if ($type == 'topic')
		{
			$replies = 0;
			$db->query("SELECT usr_group_id FROM ".$this->table_prefix."_forums WHERE id=".$fid);
			if (!$db->next_record()) return false;
			$forum_group_id = $db->f("usr_group_id");
			list($group_id, $rights) = $this->get_group_and_rights('forum_default_rights', $block_id, $forum_group_id);
			$group_id = empty($group_id) ? $forum_group_id : $group_id;
			$rights = $item_rights ? $item_rights : $rights;
		}
		else
		{
			$replies = "NULL";
			$query = 'SELECT usr_group_id, rights FROM '.$this->table_prefix.'_forum_posts WHERE id = '.$topic;
			$db->query($query);
			$db->next_record();
			$group_id = $db->f("usr_group_id");
			$rights = $db->f("rights");
			$query = "SELECT id, text FROM ".$this->table_prefix."_forum_posts WHERE
				forum_id=".$fid." AND topic_id=".$topic." AND username='".$username."'
				AND owner_id=".$owner_id." AND ip='".$ip."'
				AND post_date >= DATE_SUB(NOW(), INTERVAL  ".$CONFIG['forum_united_time']." SECOND)";

			$db->query($query);
			$db->next_record();
			$is_glue = FALSE;
			if ($db->nf()) {
				$id = $db->f('id');
				$text = $db->f('text')."\n\n".$text;
				$is_glue = TRUE;
			}
		}

		$query = 'REPLACE INTO '.$this->table_prefix.'_forum_posts (
															 id,
															 forum_id,
															 parent_id,
													 		 topic_id,
													 		 username,
													 		 email,
													 		 signature,
													 		 user_id,
													 		 subject,
													 		 text,
													 		 replies,
													 		 ip,
													 		 host,
													 		 updated,
													 		 post_date,
															 author_module,
															 author_module_link,
															 owner_id,
															 usr_group_id,
															 rights)
						VALUES ('.$id.',
								'.$fid.',
								'.$parent.',
								'.$topic.',
								"'.$username.'",
								"'.$email.'",
								"'.$signature.'",
								'.$user_id.',
								"'.$subject.'",
								"'.$text.'",
								'.$replies.',
								"'.$ip.'",
								"'.$host.'",
								NOW(),
								NOW(),
								"'.$author_module.'",
								"'.$author_module_link.'",
								'.$owner_id.',
								'.$group_id.',
								'.$rights.')';
		$db->query($query);
		$lid = $db->lid();

		if ($db->affected_rows() > 0)
		{
			$sitename = str_replace(array('=',',',';',' ',"\t","\r","\n","\013","\014"), array('','','','','','','','',''),$sitename);

			setcookie("abo-forum-username", $username, time() + 2678400);
			setcookie("abo-forum-email", $email, time() + 2678400);

			if ($type == 'reply' && !$is_glue)
			{
				$query = 'UPDATE '.$this->table_prefix.'_forum_posts SET replies = replies+1, updated = NOW() WHERE id = '.$topic;
				$db->query($query);

				/*
				// послать reply на email, если указано пользователем
				$query = "SELECT * FROM {$CONFIG["forum_posts_table"]} WHERE id = $parent";
				$db->query($query);
				$db->next_record();
				if ($CONFIG["forum_send_email"] && $db->nf() > 0 && $db->f('replies_to_email')) {
					if ($db->f('email')) {
//						$url = "$baseurl&action=showreplies&fid=$fid&topic=$topic&current=$parent&start=$start";
						$url = "$baseurl&action=showreplies&fid=$fid&topic=$topic&start=$start";
						$subject = "from $username: $subject";
						$body = "
						$text

						$url";

						$subject = convert_cyr_string($subject,"w","k");
						$body = convert_cyr_string($body,"w","k");

						@mail($db->f('email'), $subject, $body, "From:\n" . "Content-Type: text/plain; charset=\"koi8-r\"\n\n");
					}
				}
				*/
			}

			$this->update_last_reply($topic);
			$this->update_forum_post_cnt($fid);

			return $lid;
		}
		return FALSE;
	}


	// функция для получения массива навигации по форуму
	function get_forum_nav($fid, $topic, $start, $nav_array_index) {
		global $CONFIG, $baseurl, $core, $lang;

		if (!is_array($nav_array_index) && count($nav_array_index) > 0) return FALSE;
		$nav_array = array();

		$siteuser_link = Core::formPageLink($CONFIG['siteusers_page_link'], $CONFIG['siteusers_page_address'], $lang, 1);
		$siteuser_link .= ('&' == substr($link, -1) ? '' : '&');
		$forum_nav_array = array(
			"0"	=>	$baseurl.'&action=showforums',							//Список форумов
			"1"	=>	$baseurl.'&action=showtopics&fid='.$fid,				//Список тем
			"2"	=>	$baseurl.'&action=newtopic&fid='.$fid,							//Новая тема
			"3"	=>	$baseurl.'&action=newreply&fid='.$fid.'&topic='.$topic,							//Добавить сообщение
			"4"	=>	$baseurl.'&action=rules&fid='.$fid.'&start='.$start,	//Правила
			"5"	=>	$siteuser_link.'action=registration',					//Регистрация на сайте
			"6"	=>	$siteuser_link.'action=registration',					//Персональные данные
			"7"	=>	$siteuser_link.'action=logout&nocash',					//Выход
			//"Вернуться назад"		=>	'javascript:history.go(-1);'							//10
		); // массив навигации по форуму, используется в show_forum_nav()

		$i = -1;
		if (count($forum_nav_array) > 0) {
			foreach ($forum_nav_array as $nav_item => $nav_link) {
				$i++;
				if (in_array($i, $nav_array_index)) $nav_array[$nav_item] = $nav_link;
			}
		}

		return $nav_array;
	}

	// функция для вывода навигации по форуму
	function show_forum_nav($fid = 0, $topic = 0, $start = 1, $nav_array_index = array(), $forum_info = false) {
		global $CONFIG, $db, $tpl, $baseurl;
		// получим массив навигации для данной страницы форума через
		$nav_array = $this->get_forum_nav($fid, $topic, $start, $nav_array_index);
		if (!(count($nav_array) > 0)) return 0;

		$tpl->newBlock("forum_nav");
		foreach ($nav_array as $key	=> $val) {
			switch ($key) {
				case 0:
					$tpl->newBlock("nav_forum_list");
					$tpl->assign('forum_list_link', $val);
					break;

				case 1:
					$tpl->newBlock("nav_topics_list");
					$tpl->assign('topics_list_link', $val);
					break;

				case 2:
					$tpl->newBlock("nav_new_topic");
					$tpl->assign('new_topic_link', $val);
					break;

				case 3:
					$tpl->newBlock("nav_new_reply");
					$tpl->assign('new_reply_link', $val);
					break;

				case 4:
					$tpl->newBlock("nav_rules");
					$tpl->assign('rules_link', $val);
					break;

				case 5:
					$tpl->newBlock("nav_registration");
					$tpl->assign('registration_link', $val);
					break;

				case 6:
					$tpl->newBlock("nav_personal");
					$tpl->assign('personal_link', $val);
					break;

				case 7:
					$tpl->newBlock("nav_exit");
					$tpl->assign('exit_link', $val);
					break;
			}
			if ($forum_info) {
				$tpl->assign($forum_info);
			}
		}

		// если пользователь авторизован
		if ($this->required_authorization && $this->is_authorized) {
			$tpl->newBlock("nav_authorized");
			$siteuser_link = Core::formPageLink($CONFIG['siteusers_page_link'], $CONFIG['siteusers_page_address'], $lang, 1);
			$siteuser_link .= ('&' == substr($link, -1) ? '' : '&');
			$tpl->assign(array(
						 'personal_link' => $siteuser_link.'&action=registration',
						 'forum_username' => $_SESSION['siteuser']["siteusername"],
						 'exit_link' => $siteuser_link.'&action=logout&nocash'
			             ));
		}

		$tpl->gotoBlock("_ROOT");

		return 1;
	}


	function show_post_form($fid, $topic, $parent, $action, $quote = null, $edit = null)
	{
		global $CONFIG, $tpl, $tpl_path, $smiles, $db, $baseurl;

		$fid = (int) $fid;
		$topic = $topic >= 1 ? (int) $topic : "";
		$parent = $parent >= 1 ? (int) $parent : "";

		$tpl->assignInclude('post_form', $tpl_path.$this->module_name.'_post_form.html');
		$tpl->assignInclude('smiles', $tpl_path.$this->module_name.'_smiles.html');
		$tpl->prepare();
		
		$text = $subject = '';
		if ($edit) {
			$edit = (int) $edit;
			$query = 'SELECT subject, text FROM '.$this->table_prefix.'_forum_posts WHERE forum_id='.$fid.' AND id='.$edit;
			$db->query($query);
			if ($db->nf()) {
				$db->next_record();
				$text = $db->f('text');
				$subject = htmlspecialchars($db->f('subject'));
			}
		} else if ($quote){
			$quote = (int) $quote;
			$query = 'SELECT text FROM '.$this->table_prefix.'_forum_posts WHERE forum_id='.$fid.' AND id='.$quote;
			$db->query($query);
			if ($db->nf()) {
				$db->next_record();
				$text = "[QUOTE]".$db->f('text')."[/QUOTE]";
			}
		}
		
		$tpl->newBlock('block_post_form');

		$tpl->assign(array(
			"form_action" => $baseurl."&action=".$action."&nocash",
			"fid" => $fid,
			"topic" =>	$topic,
			"parent" =>	$parent,
			"subject" => $subject,
			"text" => $text,
			"id" => $edit,
			"subject_max_length" =>	$CONFIG["forum_subject_max_length"],
			"username_max_length" => $CONFIG["forum_username_max_length"],
			"email_max_length" => $CONFIG["forum_email_max_length"],
			"buttons_img_path" => "/mod/forum/img/buttons",
		));
		
		if (! $topic) {
		    $tpl->newBlock('block_subject');
		    $tpl->assign(array(
		        "subject" => $subject,
			));
		}

		$tpl->newBlock('block_'.($edit ? 'edit' :($topic ? 'reply' : 'topic')).'_title');
		if ($this->is_authorized) {
			$tpl->newBlock('block_is_authorized');
			$username = $_SESSION['siteuser']['siteusername'];
		} else {
			$tpl->newBlock('block_no_authorized');
			/*$username = $_COOKIE[$CONFIG["sitename"]."-forum-username"]
				? $_COOKIE[$CONFIG["sitename"]."-forum-username"]
				: $CONFIG['forum_default_username'];*/
			$username = $CONFIG['forum_default_username'];
		}
		$tpl->assign('username', $username);

		$smileys = $smiles->get_smiles();
		for ($i=0; $i<count($smileys); $i++) {
			$tpl->newBlock('smile');
			$tpl->assign(
				array(
					'virtual_tag' => $smiles->wrap_tag($smileys[$i]['virtual_tag']),
					'img'		  => $smiles->img_path.$smileys[$i]['img'],
					'emotion'	  => $smileys[$i]['emotion']
				)
			);
			if (!(($i+1)%$smiles->columns)) {
				$tpl->newBlock('br');
			}
		}
	}

	/*
	function show_post_form($quote = 0, $topic = false)
	{
		global $REPLACEMENTS, $CONFIG, $tpl, $tpl_path, $smiles, $db, $server, $lang;

		$tpl->assignInclude('post_form', $tpl_path.$this->module_name.'_post_form.html');
		$tpl->assignInclude('smiles', $tpl_path.$this->module_name.'_smiles.html');
		$tpl->prepare();

		$tpl->newBlock('block_'.($topic ? 'topic' : 'reply').'_title');
		if ($this->is_authorized) {
			$tpl->newBlock('block_is_authorized');
			$username = $_SESSION['siteuser']['siteusername'];
		} else {
			$tpl->newBlock('block_no_authorized');
			$username = $_COOKIE[$CONFIG["sitename"]."-forum-username"]
				? $_COOKIE[$CONFIG["sitename"]."-forum-username"]
				: $CONFIG['forum_default_username'];

		}
		$tpl->assign('username', $username);

		$smileys = $smiles->get_smiles();
		for ($i=0; $i<count($smileys); $i++) {
			$tpl->newBlock('smile');
			$tpl->assign(
				array(
					'virtual_tag' => $smiles->wrap_tag($smileys[$i]['virtual_tag']),
					'img'		  => $smiles->img_path.$smileys[$i]['img'],
					'emotion'	  => $smileys[$i]['emotion']
				)
			);
			if (!(($i+1)%$smiles->columns)) {
				$tpl->newBlock('br');
			}
		}

		$tpl->assign('_ROOT.buttons_img_path', '/mod/forum/img/buttons');

		if ($quote){
			require_once('mod/forum/buttons_config.php');
			$query = 'SELECT text FROM '.$this->table_prefix.'_forum_posts WHERE id = '.$quote;
			$db->query($query);
			if ($db->nf()){
				$db->next_record();
				$open  = str_replace('\\', '', $REPLACEMENTS['QUOTE']['open']);
				$close = str_replace('\\', '', $REPLACEMENTS['QUOTE']['close']);
				$tpl->assign(array(
					'_ROOT.quote' => $open.$db->f('text').$close
				));
			}
		}
	}
	*/

	function parse_misc($text){
		return BbCode::parse($text);
	}

	/**
	* Получить правила форума по шв
	* @param int	$id
	* @return mixed
	*/
	function get_rules($id)
	{   global $CONFIG, $db, $server, $lang;

		$id = intval($id);
		$query = 'SELECT rules FROM '.$this->table_prefix.'_forums WHERE id = '.$id;
		$db->query($query);
		if ($db->nf() > 0) {
			$db->next_record();
			return $db->f('rules');
		}

		return FALSE;
	}

	/**
	* Принициализирвать массив цензора
	*/
	function init_censor_array()
	{   global $db, $server, $lang;

		$query = 'SELECT * FROM '.$this->table_prefix.'_forum_censor';
		$db->query($query);
		while ($db->next_record()) {
			$this->censor[$db->f('censor_type')]['search'][] = ($db->f('censor_type') == 'begin'? '@\\b' : '@').$db->f('censor_word').'@i';
			$this->censor[$db->f('censor_type')]['replace'][] = $db->f('censor_replace');
		}
	}

	/**
	* Процензировать текст
	* @param string	$text
	*/
	function censor_parse(&$text){
        if(!$this->censor['middle']['search'] || !$this->censor['middle']['replace']){return;}
		$text = preg_replace($this->censor['middle']['search'], $this->censor['middle']['replace'], $text);
		$text = preg_replace($this->censor['begin']['search'], $this->censor['begin']['replace'], $text);
	}

	function getPropertyFields() {
		GLOBAL $request_site_target, $request_sub_action;
		if ('' != $request_sub_action) {
			$tpl = $this->getTmpProperties();
			switch ($request_sub_action) {
				case 'lasttopics':
			    case 'newtopics':
			        $tpl->newBlock('block_properties');
					$tpl->assign(Array(
						"MSG_Topics_per_page" => $this->_msg["Topics_per_page"],
					));
					break;

				default:
					$tpl->newBlock('block_properties_none');
					$tpl->assign(Array(
						"MSG_No" => $this->_msg["No"],
					));
					break;
			}
			return $this->getOutputContent($tpl->getOutputContent());
		}
		return FALSE;
	}

	function getEditLink($sub_action = NULL, $block_id = NULL) {
		return FALSE;
	}

	function set_category($data = NULL, $block_id = NULL)
	{
		global $CONFIG, $db;

		$data['title'] = $db->escape($data['title']);

		if (!$data['title']) {
			return false;
		}
		$data['description'] = $db->escape($data['description']);
		$data['id'] = isset($data['id']) ? (int) $data['id'] : 0;
		$data['order'] = (int) $data['order'];

		$owner_id = $_SESSION["siteuser"]["id"];
		list($group_id, $rights) = $this->get_group_and_rights('forum_default_rights', $block_id);
		$query = "REPLACE INTO ".$this->table_prefix."_forum_category"
		  ." (id, name, description, `order`, owner_id, usr_group_id, rights) VALUES"
		  ." (".$data['id'].", '".$data['title'] ."', '".$data['description']
		  ."', {$data['order']}, ".$owner_id.", ".$group_id.", ".$rights.")";
		return $db->query($query);
	}

	function get_ctg_list($mask = 0xF)
	{
		global $CONFIG, $db;

		$this->get_list_with_rights("c.*", $this->table_prefix."_forum_category AS c", "c", "", "", "c.order");
		$ret = array();
		while ($db->next_record()) {
			if ($mask & $db->f('user_rights')) {
				$ret[$db->f('id')] = $db->Record;
			}
		}

		return $ret;
	}

	function del_ctg($ctg_id)
	{
		global $db;

		$ctg_id = (int)$ctg_id;
		$db->query("SELECT COUNT(*) AS cnt FROM ".$this->table_prefix."_forums WHERE category_id=".$ctg_id);
		$db->next_record();
		if ($db->f('cnt') > 0) {
			return FALSE;
		}
		$db->query("DELETE FROM ".$this->table_prefix."_forum_category WHERE id=".$ctg_id);

		return $db->affected_rows();
	}

	function show_forum_adm($id = '') {
		global $CONFIG, $db, $baseurl;

		$id = (int)$id;
		if (!$id) return FALSE;
		$admin = Module::is_admin();
		$active_str = (!$admin) ? "AND active = 1" : "";
		$this->get_list_with_rights(
						"f.*", $this->table_prefix."_forums f", "f",
						"f.id=".$id.' '.$active_str);
		if ($db->nf() > 0) {
			$db->next_record();
			$fname			= htmlspecialchars($db->f('name'));
			$description	= htmlspecialchars($db->f('description'));
			$state =$db->f('state');
			$arr['forum_id']			= $db->f('id');
			$arr['category_id']			= $db->f('category_id');
			$arr['forum_name']			= $fname;
			$arr['forum_description']	= $description;
			$arr['forum_rules']			= $db->f('rules');
			$arr['forum_state']			= $state;
			$arr['forum_is_open']		= (0 == $state);
			$arr['forum_weight']		= $db->f('weight');
			$arr['forum_link']			= $baseurl.'&action=showtopics&fid='.$db->f('id');
			$arr['owner_id']			= $db->f('owner_id');
			$arr['user_rights']			= $db->f('user_rights');
			$arr['edit_personal_msg']	= $db->f('edit_personal_msg');
			$arr['moderators']			= $this->get_forum_moderators($id);

			return $arr;
		}
		return FALSE;
	}

	function get_ctg_adm($id = '') {
		global $CONFIG, $db, $baseurlg;

		$id = (int)$id;
		if (!$id) return FALSE;
		$admin = Module::is_admin();

		$this->get_list_with_rights("c.*", $this->table_prefix."_forum_category c", "c", "c.id=".$id);
		if ($db->nf() > 0) {
			$db->next_record();
			$arr['ctg_id'] = $db->f('id');
			$arr['ctg_name'] = htmlspecialchars($db->f('name'));
			$arr['ctg_order'] = htmlspecialchars($db->f('order'));
			$arr['ctg_description'] = htmlspecialchars($db->f('description'));
			$arr['owner_id'] = $db->f('owner_id');
			$arr['user_rights'] = $db->f('user_rights');

			return $arr;
		}
		return FALSE;
	}

	function set_forum($data, $block_id = 0)
	{
		global $CONFIG, $db;

		$data['name'] = $db->escape($data['name']);
		$data['id'] = (int) $data['id'];
		$data['moderators'] = explode(",", $data['moderators']);
		$user_list = $this->get_siteusers();
		while (list($k, $v) = each($data['moderators'])) {
			$v = (int) $v;
			if ($v > 0 && array_key_exists($v, $user_list)) {
				$data['moderators'][$k] = $v;
			} else {
				unset($data['moderators'][$k]);
			}
		}
		if (!$data['name'] || empty($data['moderators']) || $data['ctg_id'] < 1) return FALSE;

		$owner_id = $_SESSION["siteuser"]["id"];
		list($group_id, $rights) = $this->get_group_and_rights('forum_default_rights', $block_id);

		$data['id'] = isset($data['id']) ? (int) $data['id'] : 0;
		$data['description'] = $db->escape($data['description']);
		$data['rules'] = $db->escape($data['rules']);
		$data['state'] = (int) $data['state'];
		$data['weight'] = (int) $data['weight'];
		$data['edit_personal_msg'] = $data['edit_personal_msg'] ? 1 : 0;

		$db->query("SELECT MAX(f_order) as max
						FROM ".$this->table_prefix."_forums");
		if ($db->nf() > 0) {
			 $db->next_record();
			 $f_order = $db->f('max') + 1;
		} else {
			$f_order = 0;
		}

		$query = "REPLACE INTO ".$this->table_prefix."_forums";
		$query .= " (id, category_id, name, description, rules, state, weight, active,";
		$query .= " f_order, edit_personal_msg,  owner_id, usr_group_id, rights) VALUES";
		$query .= " (".$data['id'].", ".$data['ctg_id'].", '".$data['name'] ."', '".$data['description'] ."',";
		$query .= " '".$data['rules']."', ".$data['state'].", ".$data['weight'].", 1, ".$f_order.", ".$data['edit_personal_msg'].",";
		$query .= $owner_id.", ".$group_id.", ".$rights.")";
		if ($db->query($query)) {
			$lid = $data['id'] ? $data['id'] : $db->lid();
			$db->query("DELETE FROM ".$this->table_prefix."_forum_moderator WHERE forum_id=".$lid);
			$query = "INSERT INTO ".$this->table_prefix."_forum_moderator (forum_id, user_id) VALUES ";
			$query .= "(".$lid.", ".implode("), (".$lid.", ", $data['moderators']).")";
			$db->query($query);

			return true;
		}

		return false;
	}

	function get_forum_moderators($forum_id)
	{
		global $db;

		$forum_id = (int) $forum_id;
		if ($forum_id < 1) {
			return false;
		}
		$ret = array();
		$query = "SELECT U.id, U.username FROM ".$this->table_prefix."_forum_moderator AS M, core_users AS U";
		$query .= " WHERE forum_id=".$forum_id." AND user_id=U.id";
		$db->query($query);
		while ($db->next_record()) {
			$id = $db->f('id');
			$ret[$id] = array('id' => $id, 'username' => $db->f('username'));
		}

		return $ret;
	}

	// функция для добавления форума в базу данных
	function add_forum($fname, $description, $rules, $block_id = 0) {
		global $CONFIG, $db, $server, $lang;
		if (!$fname) return FALSE;

		$owner_id = $_SESSION["siteuser"]["id"];
		list($group_id, $rights) = $this->get_group_and_rights('forum_default_rights', $block_id);

		$fname			= $db->escape($fname);
		$description	= $db->escape($description);
		$rules			= $db->escape($rules);
		$db->query("SELECT MAX(f_order) as max
						FROM ".$this->table_prefix."_forums");
		if ($db->nf() > 0) {$db->next_record(); $f_order = $db->f('max') + 1;} else {$f_order = 0;}
		$query = "INSERT INTO ".$this->table_prefix."_forums (name, description, rules, f_order, owner_id, usr_group_id, rights)
					VALUES ('$fname', '$description', '$rules', $f_order, $owner_id, $group_id, $rights)";
		$db->query($query);
		if ($db->affected_rows() > 0) return $lid;
		return FALSE;
	}


	// функция для редактирования форума в базе данных
	function update_forum($fname, $description, $rules, $id) {
		global $CONFIG, $db, $server, $lang;
		$id = (int)$id;
		if (!$id || !$fname) return FALSE;
		$fname			= $db->escape($fname);
		$description	= $db->escape($description);
		$rules			= $db->escape($rules);
		$query			= "UPDATE ".$this->table_prefix."_forums SET name = '$fname', description = '$description', rules = '$rules' WHERE id = $id";
		$db->query($query);
		return TRUE;
	}

	function update_reply($id, $subject, $text){
		global $CONFIG, $db, $server, $lang;

		$subject	= $db->escape($subject);
		$text		= $db->escape($text);
		$reply = $this->get_topic($id);

		if (!$reply) {
			return false;
		}

		// Пользователь должен быть зарегистрирован.
		if (!isset($_SESSION['siteuser']["id"]) || !intval($_SESSION['siteuser']["id"]))
		{
			return false;
		}

		$query = 'UPDATE '.$this->table_prefix.'_forum_posts SET '
		    .($subject ? 'subject = "'.$subject.'", ' : '')
		    .'text = "'.$text.'", updated=NOW(), edit_user_id='.intval($_SESSION['siteuser']["id"])
		    .', edit_username="'.$_SESSION['siteuser']["siteusername"].'" WHERE id = '.$id;
		$db->query($query);

		return $db->affected_rows();
	}

	// функция для удаления форума из базы данных
	function delete_forum($id) {
		global $CONFIG, $db, $server, $lang;
		$id = (int)$id;
		if (!$id) return FALSE;
		$query = "DELETE FROM ".$this->table_prefix."_forums WHERE id=".$id;
		$db->query($query);
		if ($db->affected_rows() > 0) {
			$db->query("SELECT id FROM ".$this->table_prefix."_forum_posts WHERE forum_id=".$id);
			$ids = array();
			while ($db->next_record()) {
				$ids[] = $db->f('id');
			}
			if (! empty($ids)) {
				$db->query("DELETE FROM ".$this->table_prefix."_forum_post_state WHERE topic_id IN (".implode(", ", $ids).")");
				$db->query("DELETE FROM ".$this->table_prefix."_forum_complaint WHERE post_id IN (".implode(", ", $ids).")");
			}
			$db->query("DELETE FROM ".$this->table_prefix."_forum_posts WHERE forum_id=".$id);
			$db->query("DELETE FROM ".$this->table_prefix."_forum_moderator WHERE forum_id=".$id);
			$db->query("DELETE FROM ".$this->table_prefix."_forum_last_reply WHERE forum_id=".$id);
			$db->query("DELETE FROM ".$this->table_prefix."_forum_moderator WHERE forum_id=".$id);
			$db->query("DELETE FROM ".$this->table_prefix."_forum_online_user WHERE forum_id=".$id);
			$db->query("DELETE FROM ".$this->table_prefix."_forum_topic_visit WHERE forum_id=".$id);

			return TRUE;
		}
		return TRUE;
	}


	// функция для удаления темы по id
	function delete_topic($topic = "", $fid = "") {
		global $db;
		$fid	= (int)$fid;
		$topic	= (int)$topic;
		if (!$topic) return FALSE;
		$db->query("SELECT id, forum_id FROM ".$this->table_prefix."_forum_posts WHERE id=".$topic." OR topic_id=".$topic);
		$ids = array();
		while ($db->next_record()) {
			$ids[] = $db->f('id');
		}
		if (! count($ids)) {
			return FALSE;
		}
		$fid = $db->f('forum_id');
		$db->query("DELETE FROM ".$this->table_prefix."_forum_posts WHERE id=".$topic." OR topic_id=".$topic);
		$db->query("DELETE FROM ".$this->table_prefix."_forum_post_state WHERE topic_id=".$topic);
		$db->query("DELETE FROM ".$this->table_prefix."_forum_post_state WHERE topic_id=".$topic);
		$db->query("DELETE FROM ".$this->table_prefix."_forum_online_user WHERE topic_id=".$topic);
		$db->query("DELETE FROM ".$this->table_prefix."_forum_topic_visit WHERE topic_id=".$topic);
		$db->query("DELETE FROM ".$this->table_prefix."_forum_complaint WHERE post_id IN (".implode(", ", $ids).")");
		$this->update_last_reply($topic);
		$this->update_forum_post_cnt($fid);

		return TRUE;
	}


	// функция для удаления сообщения по id
	function delete_reply($id)
	{
		global $CONFIG, $db;

		$id		= (int)$id;
		if (!$id) {
			return FALSE;
		}

		$db->query("SELECT topic_id, forum_id FROM ".$this->table_prefix."_forum_posts WHERE id=".$id);
		if (! $db->nf()) {
			return FALSE;
		}
		$db->next_record();
		$topic_id = $db->f('topic_id');
		$fid = $db->f('forum_id');

		$db->query("DELETE FROM ".$this->table_prefix."_forum_posts WHERE id=".$id);
		$db->query("DELETE FROM ".$this->table_prefix."_forum_complaint WHERE post_id=".$id);


		$db->query("UPDATE ".$this->table_prefix."_forum_posts SET replies = (replies - 1) WHERE id=".$topic_id);
		$this->update_last_reply($topic_id);
		$this->update_forum_post_cnt($fid);

		return TRUE;
	}

	/**
	* Обновит тему
	* @param int	$id	Id темы
	* @param string	$subject	Заголовок сообжения
	* @param string	$text	Текс сообжения
	* @return void
	*/
	function update_topic($id, $subject, $text, $type = 0, $status = 0)
	{
		global $CONFIG, $db, $server, $lang;

		$subject	= $db->escape($subject);
		$text		= $db->escape($text);
		$type 		= (int) $type;
		$type 		= (int) $type;
		$topic = $this->get_topic($id);

		if (!$topic || !$subject) {
			return false;
		}

		$set = $topic['is_owner']
			? ""
			: ", edit_user_id = ".$_SESSION['siteuser']["id"].", edit_username = '".$_SESSION['siteuser']["siteusername"]."'";
        $query = 'UPDATE `'.$this->table_prefix.'_forum_posts` SET text = "'.$text.'", subject = "'.$subject.'"'.$set.' WHERE id = '.$id;

        $db->query($query);

        $query = 'REPLACE INTO '.$this->table_prefix.'_forum_post_state (topic_id, type, status) VALUES ('.$id.','.$type.','.$status.')';
		$db->query($query);

	}

	// функция для перемещения тем в форум fid
	function move_topics($ids, $fid)
	{
		global $CONFIG, $db, $server, $lang;

		$id = intval($fid);
		if (!$id && !is_array($ids)) return false;

		$query = 'UPDATE '.$this->table_prefix.'_forum_posts SET forum_id = '.$fid;
		$query .= ' WHERE id IN ('.implode(",", $ids).') OR topic_id IN ('.implode(",", $ids).')';
		$db->query($query);
		$query = 'UPDATE '.$this->table_prefix.'_forum_last_reply SET forum_id = '.$fid;
		$query .= ' WHERE reply_id IN ('.implode(",", $ids).') OR topic_id IN ('.implode(",", $ids).')';
		$db->query($query);
		$query = 'UPDATE '.$this->table_prefix.'_forum_online_user SET forum_id = '.$fid;
		$query .= ' WHERE topic_id IN ('.implode(",", $ids).')';
		$db->query($query);
		$query = 'UPDATE '.$this->table_prefix.'_forum_topic_visit SET forum_id = '.$fid;
		$query .= ' WHERE topic_id IN ('.implode(",", $ids).')';
		$db->query($query);
		$this->update_forum_post_cnt($id);
	}

	// функция для перемещения сообщений в тему
	function move_replies($ids, $topic_id)
	{
		global $db;

		$topic_id = intval($topic_id);
		if ($topic_id < 1 || !is_array($ids)) {
			return false;
		}
		/* Есть ли такая тема*/
		$db->query('SELECT id FROM '.$this->table_prefix.'_forum_posts WHERE id = '.$topic_id.' AND topic_id=0');
		if (! $db->nf()) {
			return FALSE;
		}
		$new_ids = array();
		foreach ($ids as $key => $val) {
			if ($val >= 1) {
				$new_ids[] = (int) $val;
			}
		}
		$ids_str = "";
		$tip = 0;
		$moved = 0;
		$query = 'SELECT id, topic_id FROM '.$this->table_prefix.'_forum_posts ';
		$query .= 'WHERE id IN ('.implode(",", $ids).') AND topic_id!=0';
		$db->query($query);
		/* псключаем из списка темы и проверяем, принадлежат ли сообщения одной теме */
		while ($db->next_record()) {
			/* Первая запись */
			if (0 == $tip) {
				$tip = $db->f('topic_id');
			}
			if ($tip != $db->f('topic_id')) {
				return FALSE;
			}
			$ids_str .= $db->f('id').",";
			++$moved;
		}
		if (0 == $moved) {
			return FALSE;
		}

		$ids_str = substr($ids_str, 0, -1);
		$query = 'UPDATE '.$this->table_prefix.'_forum_posts SET topic_id = '.$topic_id;
		$query .= ' WHERE id IN ('.$ids_str.')';
		$db->query($query);
		/* Обновляем кол-во сообщений в теме */
		$db->query('UPDATE '.$this->table_prefix.'_forum_posts SET replies=replies-'.$moved.' WHERE id='.$tip);
		$db->query('UPDATE '.$this->table_prefix.'_forum_posts SET replies=replies+'.$moved.' WHERE id='.$topic_id);
		/* Обновляем информацию о последнем сообщений в теме */
		$this->update_last_reply($topic_id);
		$this->update_last_reply($tip);

		return TRUE;
	}

	// функция для вывода списка сообщений для журнала
	function show_log($start_date, $end_date, $rows, $start, $view = "") {
		global $CONFIG, $main, $core, $db, $db1, $db2, $tpl, $baseurl, $server, $lang, $smiles;

		$blog_link = $core->formPageLink($CONFIG["blogs_page_link"], $CONFIG["blogs_page_address"], $lang);

		$admin = Module::is_admin();
    	if (!$rows) $rows = $CONFIG["forum_replies_max_rows"];				// выводить строк
    	$order_by = "post_date DESC";										// порядок вывода
		$start = ((int)$start) ? $start : 1;								// выводить начиная со страницы

		$total_cnt = $this->get_list_with_rights(
								'P.*,
								DATE_FORMAT(post_date,"'.$CONFIG['forum_date_format'].'") as postdate,
								DATE_FORMAT(updated,"'.$CONFIG['forum_date_format'].'") as topic_updated, IB.ip as ip_ban',
								$this->table_prefix."_forum_posts AS P LEFT JOIN ".$this->table_prefix."_forum_ip_ban IB ON P.ip = IB.ip", "P",
								'post_date >= "'.date("Y-m-d", $start_date).'" AND post_date <= "'.date("Y-m-d 23:59:59", $end_date).'"
								AND ( topic_id > 0 OR replies > 0)',
								"", $order_by,
								$start, $rows,
								true);
		$pages_cnt = ceil($total_cnt/$rows);

		$cnt = $db->nf();
		if ($cnt > 0) {
			// вывод на экран списка
			for ($i=0; $i < $cnt; $i++) {
				$db->next_record();

				$user_id = $db->f('user_id');
				$username = $db->f('username');

				$user_info = $user_id
					? '<a href="/admin.php?lang='.$lang.'&name=siteusers&action=editsiteuser&id='.$user_id.'" title="пнформация о пользователе">'.$username.'</a>'
					: $username;

				$signature	= ($db->f('signature')) ? "<hr>" . nl2br($db->f('signature')) : "";
				$id			= $db->f('id');
				$bgcolor	= ($bgcolor == "#FFFFFF" || $bgcolor == "") ? "#FFFFFF" : "#FFFFFF";

				$text = $db->f('text');
				$text = str_replace('<', '&lt;', $text);
				$text = str_replace('>', '&gt;', $text);
				$text = $smiles->parse($text);
				$text = $this->parse_misc($text);

				$subject = $db->f('subject');
				$postdate = $db->f('postdate');
				$topic_updated = $db->f('topic_updated');
				$new_message = $db->f('new_message');
				$ip = $db->f('ip');
				$ip_ban = $db->f('ip_ban');

				$act_type = $db->f('topic_id') ? 'reply' : 'topic';
				$link_end = $db->f('topic_id') ? '&topic='.$db->f('topic_id').'&id='.$id : '&topic='.$id.'&id='.$id;
				$tpl->newBlock('block_row');

				$actions = array(	"edit"		=> "edit".$act_type,
									"delete"	=> "del".$act_type,
									"confirm_delete"	=> $this->_msg["Confirm_delete_this_message"]
								);
				$adm_actions = $this->get_actions_by_rights($baseurl."&fid=".$db->f('forum_id').$link_end."&start=".$start, $db->f("active"), $db->f("is_owner"), $db->f("user_rights"), $actions);

				$tpl->assign(
					array(
						"MSG_Ban_IP"	=> $this->_msg["Ban_IP"],
						"MSG_Title"		=> $this->_msg["Title"],
						"subject"			=> $subject,
						"text"			=> $text,
						"signature"		=> $signature,
						"username"		=> $username,
						"user_info"		=> $user_info,
						"user_link"		=> $baseurl.'&action=userinfo&id='.$user_id,
						"post_date"		=> $postdate,
						"updated"			=> $topic_updated,
						"bgcolor"			=> $bgcolor,
						"new_message"		=> ($new_message) ? '<span class=fnew>новое!</span>&nbsp;&nbsp;' : '',
						"reply_del_action"=> $adm_actions["delete_action"],
						"reply_edit_action"=> $adm_actions["edit_action"],
					)
				);
				if ($db->f("is_owner") && empty($ip_ban)) {
					$tpl->newBlock('block_ip_ban');
					$tpl->assign('banip_link', $baseurl.'&action=banip&ip='.$ip);
				}

				$author_module = $db->f('author_module');
				if ($author_module) {
					$mod_link = Core::formPageLink($CONFIG[$author_module.'_page_link'], $CONFIG[$author_module.'_page_address'], $lang, 1);
					$mod_link .= $db->f('author_module_link');
					$tpl->newBlock('block_author_module');
					$tpl->assign(array(
						'author_module_link' => $mod_link,
						'site_name'			 => $_SERVER["HTTP_HOST"],
					));
				}
			}
			return $pages_cnt;
		}
		return FALSE;
	}

		// функция для вывода списка сообщений для журнала
	function show_complaints($start_date, $end_date, $rows, $start, $view = "") {
		global $CONFIG, $main, $core, $db, $db1, $db2, $tpl, $baseurl, $server, $lang, $smiles;

		if (!$rows) $rows = $CONFIG["forum_replies_max_rows"];				// выводить строк
    	$order_by = "cdate DESC";											// порядок вывода
		$start = ((int)$start) ? $start : 1;								// выводить начиная со страницы

		$total_cnt = $this->get_list_with_rights(
								'C.user_id as cmpl_user_id, C.username as cmpl_username,
									DATE_FORMAT(C.cdate,"'.$CONFIG['forum_date_format'].'") as cmpl_date,
									P.*,
									DATE_FORMAT(post_date,"'.$CONFIG['forum_date_format'].'") as postdate,
									DATE_FORMAT(updated,"'.$CONFIG['forum_date_format'].'") as topic_updated,
									IB.ip as ip_ban',
								$this->table_prefix."_forum_complaint AS C, ".$this->table_prefix."_forum_posts AS P
									LEFT JOIN ".$this->table_prefix."_forum_ip_ban IB ON P.ip = IB.ip",
								"P",
								'C.post_id=P.id AND C.cdate >= "'.date("Y-m-d", $start_date).'" AND C.cdate <= "'.date("Y-m-d 23:59:59", $end_date).'"
									AND ((forum_id NOT IN ('.$CONFIG['news_forum_id_for_discuss'].','.$CONFIG['catalog_forum_id_for_discuss'].'))
									OR (topic_id > 0) OR (replies > 0))',
								"",
								$order_by,
								$start,
								$rows,
								true);
		$pages_cnt = ceil($total_cnt/$rows);

		$cnt = $db->nf();
		if ($cnt > 0) {
			// вывод на экран списка
			for ($i=0; $i < $cnt; $i++) {
				$db->next_record();

				$user_id = $db->f('user_id');
				$username = $db->f('username');

				$user_info = $user_id
					? '<a href="/admin.php?lang='.$lang.'&name=siteusers&action=editsiteuser&id='.$user_id.'" title="пнформация о пользователе">'.$username.'</a>'
					: $username;

				$signature	= ($db->f('signature')) ? "<hr>" . nl2br($db->f('signature')) : "";
				$id			= $db->f('id');
				$bgcolor	= ($bgcolor == "#FFFFFF" || $bgcolor == "") ? "#FFFFFF" : "#FFFFFF";

				$text = $db->f('text');
				$text = str_replace('<', '&lt;', $text);
				$text = str_replace('>', '&gt;', $text);
				$text = $smiles->parse($text);
				$text = $this->parse_misc($text);

				$subject = $db->f('subject');
				$postdate = $db->f('postdate');
				$topic_updated = $db->f('topic_updated');
				$new_message = $db->f('new_message');
				$ip = $db->f('ip');
				$ip_ban = $db->f('ip_ban');
				$cmpl_user_id = $db->f('cmpl_user_id');
				$cmpl_username = $db->f('cmpl_username');
				$cmpl_date = $db->f('cmpl_date');

				$act_type = $db->f('topic_id') ? 'reply' : 'topic';
				$link_end = $db->f('topic_id') ? '&topic='.$db->f('topic_id').'&id='.$id : '&topic='.$id.'&id='.$id;
				$tpl->newBlock('block_row');

				$actions = array(	"edit"		=> "edit".$act_type,
									"delete"	=> "del".$act_type,
									"confirm_delete"	=> $this->_msg["Confirm_delete_this_message"]
								);
				$adm_actions = $this->get_actions_by_rights($baseurl."&fid=".$db->f('forum_id').$link_end."&start=".$start, $db->f("active"), $db->f("is_owner"), $db->f("user_rights"), $actions);

				$tpl->assign(
					array(
						"MSG_Ban_IP"	=> $this->_msg["Ban_IP"],
						"MSG_Title"		=> $this->_msg["Title"],
						"subject"			=> $subject,
						"text"			=> $text,
						"signature"		=> $signature,
						"username"		=> $username,
						"user_info"		=> $user_info,
						"user_link"		=> $baseurl.'&action=userinfo&id='.$user_id,
						"post_date"		=> $postdate,
						"updated"			=> $topic_updated,
						"bgcolor"			=> $bgcolor,
						"new_message"		=> ($new_message) ? '<span class=fnew>новое!</span>&nbsp;&nbsp;' : '',
						"reply_del_action"=> $adm_actions["delete_action"],
						"reply_edit_action"=> $adm_actions["edit_action"],
					)
				);
				$tpl->newBlock('block_complaint');
				$tpl->assign(
					array(
						'user_link' => '/admin.php?lang='.$lang.'&name=siteusers&action=editsiteuser&id='.$cmpl_user_id,
						'username'	=> $cmpl_username,
						'date'	=> $cmpl_date,
					)
				);
				if ($db->f("is_owner") && empty($ip_ban)) {
					$tpl->newBlock('block_ip_ban');
					$tpl->assign('banip_link', $baseurl.'&action=banip&ip='.$ip);
				}

				$author_module = $db->f('author_module');
				if ($author_module) {
					$mod_link = Core::formPageLink($CONFIG[$author_module.'_page_link'], $CONFIG[$author_module.'_page_address'], $lang, 1);
					$mod_link .= $db->f('author_module_link');
					$tpl->newBlock('block_author_module');
					$tpl->assign(array(
						'author_module_link' => $mod_link,
						'site_name'			 => $_SERVER["HTTP_HOST"],
					));
				}
			}
			return $pages_cnt;
		}
		return FALSE;
	}

	/**
	* Отображение забаненых IP
	*/
	function show_banlist ($start)
	{   global $db, $server, $lang, $CONFIG, $tpl, $baseurl;

		$query = 'SELECT * FROM '.$this->table_prefix.'_forum_ip_ban
					ORDER BY ip
					LIMIT '.(($start-1)*$CONFIG["forum_banip_max_rows"]).','.$CONFIG["forum_banip_max_rows"];
		$db->query($query);
		while ($db->next_record()) {
			$tpl->newBlock("block_row");
			$tpl->assign(Array(
				"MSG_Confirm_move_from_banlist" => $this->_msg["Confirm_move_from_banlist"],
				"MSG_Delete" => $this->_msg["Delete"],
			));
			$tpl->assign(array(
							"ip"			=> $db->f('ip'),
							"ip_del_link"	=> $baseurl."&action=delbanip&ip=".$db->f('ip')."&start=".$start,
						));
		}
	}

	/**
	* Добавление забаненного IP
	*/
	function ban_ip ($ip)
	{   global $db, $server, $lang;

		$query = 'REPLACE INTO '.$this->table_prefix.'_forum_ip_ban SET ip="'.$ip.'"';
		$db->query($query);
	}

	// преобразование даты
	function test_date($date) {
		$date_arr = explode(".", $date);
		if (sizeof($date_arr) != 3)  return null;
		if (intval($date_arr[0]) < 0 || intval($date_arr[0]) > 31)  return null;
		if (intval($date_arr[1]) < 0 || intval($date_arr[1]) > 12)  return null;
		if (intval($date_arr[2]) == 0)  return "";
		return mktime(0,0,0, $date_arr[1], $date_arr[0], $date_arr[2]);
	}

	/**
	* Регистриция посещения пользователем форума + удаление устаревших посещений
	*
	* @param int	$forum_id	Id форума
	* @param int	$topic_id	Id темы
	* @return void
	**/
	function reg_forum_vizit($forum_id, $topic_id)
	{
		global $db, $CONFIG;

		$forum_id = (int) $forum_id;
		$topic_id = (int) $topic_id;
		$user_id = isset($_SESSION['siteuser']["id"]) ? (int) $_SESSION['siteuser']["id"] : 0;
		$username = isset($_SESSION['siteuser']["siteusername"]) ?  $db->escape($_SESSION['siteuser']["siteusername"]) : "";

		$query = "REPLACE INTO ".$this->table_prefix."_forum_online_user";
		$query .= " (sid, vtime, forum_id, topic_id, user_id, username) VALUES (";
		$query .= "'".session_id()."', NOW(), ".$forum_id.", ".$topic_id.", ";
		$query .= $user_id.", '".$username."')";
		$db->query($query);

		$db->query("DELETE FROM ".$this->table_prefix."_forum_online_user WHERE vtime < DATE_SUB(NOW(), INTERVAL ".$CONFIG['forum_active_time']." MINUTE)");
	}

	/**
	* Обновить параметры последнего сообщения темы
	*
	* @param int	$topic_id	Id темы
	* @return boolean
	**/
	function update_last_reply($topic_id)
	{
		global $db;

		$topic_id = (int) $topic_id;
		if ($topic_id < 1)
		{
			return false;
		}

		$query = "SELECT topic_id, forum_id, id, post_date, user_id, username";
		$query .= " FROM ".$this->table_prefix."_forum_posts";
		$query .= " WHERE topic_id=".$topic_id." OR id=".$topic_id." ORDER BY post_date DESC LIMIT 1";
		$db->query($query);

		if ($db->nf() > 0)
		{
			$db->next_record();
			$forum_id = $db->f('forum_id');
			$reply_id = $db->f('id');
			$post_date = $db->f('post_date');
			$user_id = $db->f('user_id');
			$username = $db->f('username');
			if (0 != $db->f('topic_id')) {
				$query = "REPLACE INTO ".$this->table_prefix."_forum_last_reply";
				$query .= " (topic_id, forum_id, reply_id, post_date, user_id, username) VALUES ";
				$query .= " (".$topic_id.", ".$forum_id.", ".$reply_id.", '".$post_date."', ";
				$query .= $user_id.", '".$username."')";
				$db->query($query);
			}
			$query = "UPDATE ".$this->table_prefix."_forums";
			$query .= " SET last_post_id=".$reply_id.", last_post_date='".$post_date."', last_user_id=".$user_id.",";
			$query .= " last_username='".$username."' WHERE id=".$forum_id;
			$db->query($query);

			return TRUE;
		}

		return FALSE;
	}

	function update_forum_post_cnt($forum_id = 0)
	{
		global $db;
		$forum_id = (int) $forum_id;
		$forums = array();
		if ($forum_id) {
			$forums[] = $forum_id;
		} else {
			$db->query("SELECT id FROM ".$this->table_prefix."_forums");
			while ($db->next_record()) $forums[] = $db->f('id');
		}
		foreach ($forums as $idx => $forum_id) {
			$topics = $replys = 0;
			$query = "SELECT COUNT(*) AS cnt FROM ".$this->table_prefix."_forum_posts";
			$query .= " WHERE forum_id=".$forum_id." AND topic_id=0";
			$db->query($query);
			$db->next_record();
			$topics = $db->f('cnt');

			$query = "SELECT COUNT(*) AS cnt FROM ".$this->table_prefix."_forum_posts";
			$query .= " WHERE forum_id=".$forum_id." AND topic_id!=0";
			$db->query($query);
			$db->next_record();
			$replys = $db->f('cnt') + $topics;

			$query = "UPDATE ".$this->table_prefix."_forums ";
			$query .= "SET topics=".$topics.", replies=".$replys;
			$query .= " WHERE id=".$forum_id;
			$db->query($query);
		}

		return $db->affected_rows();
	}

	function show_moderators($moderators, $block_name = 'block_moderator')
	{
		global $CONFIG, $tpl;

		if (! is_array($moderators)) {
			return FALSE;
		}
		Module::load_mod('siteusers', 'only_create_object');
		foreach ($moderators as $key => $val) {
			$tpl->newBlock($block_name);
			$tpl->assign(
				array(
					'id' => $val['id'],
					'username' => $val['username'],
					'user_link' => SiteusersPrototype::getUserLink($val['username']),
				)
			);
		}
	}

	function showFilterForm($forum_id, $topic_id, $start, $filter = array(), $block_name = 'block_filter_form')
	{
		global $tpl, $baseurl, $PAGE, $lang;
		$tpl->newBlock($block_name);
		$tpl->assign(
			array(
				'fid' => $forum_id,
				'topic' => $topic_id,
				'start' => $start,
				'lang' => $lang,
				'link' => $PAGE['link'],
				'filter_topics_action' => $baseurl."&action=showtopics&fid=".$forum_id."&start=".$start,
				'filter_replies_action' => $baseurl."&action=showreplies&fid=".$forum_id."&topic=".$topic_id."&start=".$start,
			)
		);
		$this->show_select('block_show_for_period', $filter['period'], 'title', $this->show_for_period);
		$this->show_select('block_order_by', $filter['order'], 'title', $this->replies_order_by);
	}



	function var_to_url_query($var, $array_name = '', $pref = "")
	{
		if (is_scalar($var)) {
			return $pref . urlencode($var);
		}
		if (is_array($var)) {
			$ret = array();
			foreach ($var as $key => $val) {
				if ($array_name) {
					$ret[] = urlencode($array_name) . "[" . urlencode($key) . "]" . "=". urlencode($val);
				} else {
					$ret[] = urlencode($key) . "=". urlencode($val);
				}
			}
			return $pref . implode("&", $ret);
		}

		return '';
	}

	function registr_complaint($post_id)
	{
		global $db, $lang, $CONFIG;

		if (! $this->is_authorized) {
			return false;
		}

		$post_id = (int) $post_id;
		$db->query("SELECT id, forum_id, topic_id, post_date FROM ".$this->table_prefix."_forum_posts WHERE id=".$post_id);
		if (! $db->nf())
		{
			return false;
		}
		$db->next_record();
		$forum_id = $db->f('forum_id');
		$topic_id = $db->f('topic_id');
		$post_date = $db->f('post_date');

		$db->query("SELECT cdate FROM ".$this->table_prefix."_forum_complaint WHERE post_id="
		    .$post_id." AND user_id=".$_SESSION['siteuser']["id"]);
		if ($db->nf())
		{
			return false;
		}

		require_once (RP.'/mod/siteusers/lib/class.Siteusers.php');
		$su = new Siteusers('only_create_object');
		$mail_tpl = file_get_contents(RP."/mod/forum/tpl/mail/complaint_".$lang.".html");
		$modrt_list = $this->get_forum_moderators($forum_id);

		foreach ($modrt_list as $key => $val) {
		    $text = str_replace(
		        array("{site}", "{usrerame}", "{post_link}", "{date}"),
		        array($CONFIG['web_address'], $_SESSION['siteuser']["siteusername"], $_SERVER['HTTP_REFERER'], $post_date),
		        $mail_tpl
		    );

			$su->send_personal_msg($_SESSION['siteuser']["id"], $val['id'], $text, true);
		}
		$db->query("INSERT INTO ".$this->table_prefix."_forum_complaint (post_id, user_id, cdate, username)
			VALUES (".$post_id.", ".$_SESSION['siteuser']["id"].",
			NOW(), '".$_SESSION['siteuser']["siteusername"]."')"
		);

		return $db->affected_rows() ? true : false;
	}
}
?>